//--
/********************************************************************************
* Plotter grid for drawing function or data plots								*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* Grid.c																		*
*																				*
*																				*
* Version:	V1.03																*
* Date:		09.11.2002															*
********************************************************************************/


#include "graphlib.h"

#define MAXOLDGRIDLIMITS	16	/* number of saved gridlimits for undo stack */

#ifdef	MEMORYGRID

/* empty grid in memory, used internaly */

extern byte FAR	GRIDMEM[];

#else /* MEMORYGRID */

/* used internaly for scaling */

#define NUMSCALEFACTORS		10

typedef struct TSCALEFACTOR {
	double	Limit;
	double	Factor;
} tScaleFactor;

typedef struct TGRIDSCALING
{
	double	ScaleStartx;
	double	ScaleDiffx;
	double	ScaleStarty;
	double	ScaleDiffy;
	INT16	ScaleNumx;
	INT16	ScaleNumy;
} tGridScaling;

extern tScaleFactor FAR	ScaleFactors[];

extern tGridScaling	FAR	GridScaling;

extern INT16			CalcScaling (double rangestart, double rangeend, double rangediff, double FAR *ScaleStart, double FAR *ScaleDiff, INT16 FAR *ScaleNum);

#endif /* MEMORYGRID */


/* for drawing a cross */
#define CROSSSIZE		5							/* size of the cross */
extern GRAPHCOORDTYPE	CrossInGridLastX;			/* Grid Position of last cross X */
extern GRAPHCOORDTYPE	CrossInGridLastY;			/* Grid Position of last cross Y */

/* for old GridLimits */
typedef struct TOLDGRIDLIMITS
{
	double	x_min;
	double	x_max;
	double	y_min;
	double	y_max;
} tOldGridLimits;

/* save old range used in LIFO */
extern void				SaveOldDisplayValues (void);

extern INT8				NumOldGridLimit;
extern INT8				LastOldGridLimit;
extern tOldGridLimits	OldGridLimits[MAXOLDGRIDLIMITS];

/* exported data */

//-GridLimits.c
/****************************************************************************************************
* GLOBAL variables
****************************************************************************************************/
tGridLimits		GridLimits =	{	DEF_XMIN,
									DEF_XMAX,
									DEF_YMIN,
									DEF_YMAX,
									DEF_XMAX - DEF_XMIN,
									DEF_YMAX - DEF_YMIN
								};
//-oldGridLimits.c
INT8			NumOldGridLimit		= 0;
INT8			LastOldGridLimit	= MAXOLDGRIDLIMITS-1;
tOldGridLimits	OldGridLimits[MAXOLDGRIDLIMITS];

//-ScaleFactors.c
#ifndef MEMORYGRID
tScaleFactor FAR	ScaleFactors[] =				{	/* for grid calculation, 2.5 to 3.5 scales */	
														{ 0.000,	0.40},		/*  1.000 -  1.400 */
														{ 1.250,	0.50},		/*  1.250 -  1.750 */
														{ 1.750,	0.60},		/*  1.500 -  2.100 */
														{ 1.875,	0.75},		/*  1.875 -  2.625 */
														{ 2.500,	1.00},		/*  2.500 -  3.500 */
														{ 3.500,	1.25},		/*  3.125 -  4.375 */
														{ 3.750,	1.50},		/*  3.750 -  5.250 */
														{ 5.000,	2.00},		/*  5.000 -  7.000 */
														{ 7.000,	2.50},		/*  6.250 -  8.750 */
														{ 8.750,	3.00}		/*  7.500 - 10.500 */
													};	
#endif /* MEMORYGRID */

//-GridScaling.c
#ifndef MEMORYGRID
tGridScaling FAR	GridScaling;
#endif /* MEMORYGRID */

//-GraphMem.c
#ifdef MEMORYGRID
byte FAR			GraphMem[GRAPHICSIZE(GRID_DX+1,GRID_DY+1)];		/* Mem for drawing Grid*/
#else /* MEMORYGRID */
tGraphMem FAR		GraphMem;
#endif /* MEMORYGRID */


//-GRIDMEM.c
#ifdef MEMORYGRID
byte FAR			GRIDMEM[] =  /* empty Grid data, local */
{
	GSIZE(145, 137),
	0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x80,
	0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x00
};
#endif /* MEMORYGRID */


//-GridxValues.c
double			GridxValues[GRID_DX+1];				/* Array of x-Values for each grid point */

//-GridxValuesExt.c
double			GridxValuesExt[(GRID_DX+1)+2];		/* Array of x-Values for each grid point */

//-CrossInGridLastXY.c
GRAPHCOORDTYPE	CrossInGridLastX		= -1;		/* Grid Position of last cross X */
GRAPHCOORDTYPE	CrossInGridLastY		= -1;		/* Grid Position of last cross Y */

//-dashon.c
unsigned INT8	dashon					= 1;		/* for dashed draw of lines */


/* exported functions */

/****************************************************************************************************
* Calc Adress of byte with coordinate dot in
****************************************************************************************************/

//-CalcXYMemAdress.c
byte FAR *CalcXYMemAdress (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
#ifdef MEMORYGRID
		return ((GraphMem + 4) + (y * XBYTES(GRID_DX+1) + (x>>3)));
#else
		return ((byte FAR *)(GraphMem.Mem) + (y * XBYTES(GRID_DX+1) + (x>>3)));
#endif
}


/****************************************************************************************************
* Calc Values in GridxValues for each dot
****************************************************************************************************/

//-CalcGridxValues.c
GRAPHCOORDTYPE	CalcGridxValues(tGridLimits *GridLimits)
{
	GRAPHCOORDTYPE x = GRID_DX+1;

	while(x--) {
		GridxValues[x] = (GridLimits->x_max * x + GridLimits->x_min * (GRID_DX - x)) / GRID_DX;
	}

	return ((GRAPHCOORDTYPE)(GRID_DX+1));
}


/****************************************************************************************************
* Calc Values in GridxValues for each dot + one aditional point before and after limits
****************************************************************************************************/

//-CalcGridxValuesExt.c
GRAPHCOORDTYPE	CalcGridxValuesExt(tGridLimits *GridLimits)
{
	GRAPHCOORDTYPE n = (GRID_DX+1)+2;
	GRAPHCOORDTYPE x = GRID_DX+1;

	while(n--) {
		GridxValuesExt[n] = (GridLimits->x_max * x + GridLimits->x_min * (GRID_DX - x)) / GRID_DX;
		x--;
	}

	return ((GRAPHCOORDTYPE)((GRID_DX+1)+2));
}


/****************************************************************************************************
* Init display values
****************************************************************************************************/

//-InitDisplayValues.c
void InitDisplayValues (double x_min, double x_max, double y_min, double y_max)
{
		GridLimits.x_min =		x_min;
		GridLimits.x_max =		x_max;
		GridLimits.x_diff =		x_max - x_min;
		GridLimits.y_min =		y_min;
		GridLimits.y_max =		y_max;
		GridLimits.y_diff =		y_max - y_min;

		NumOldGridLimit = 0;
}


/****************************************************************************************************
* Set display values
****************************************************************************************************/

//-SetDisplayValues.c
INT8 SetDisplayValues (double x_min, double x_max, double y_min, double y_max)
{
	if((x_max > x_min) && (y_max > y_min))
	{
		SaveOldDisplayValues ();

		GridLimits.x_min =		x_min;
		GridLimits.x_max =		x_max;
		GridLimits.x_diff =		x_max - x_min;
		GridLimits.y_min =		y_min;
		GridLimits.y_max =		y_max;
		GridLimits.y_diff =		y_max - y_min;

		return ((INT8)TRUE);
	}

	return ((INT8)FALSE);
}


/****************************************************************************************************
* Save old display values for undo scaling
****************************************************************************************************/

//-SaveOldDisplayValues.c
void SaveOldDisplayValues (void)
{
	memcpy(&(OldGridLimits[(LastOldGridLimit = (LastOldGridLimit + 1) % MAXOLDGRIDLIMITS)]), &GridLimits, sizeof(tOldGridLimits));

	if(NumOldGridLimit < MAXOLDGRIDLIMITS) NumOldGridLimit++;
}


/****************************************************************************************************
* Restore old display values for undo scaling
****************************************************************************************************/

//-RestoreOldDisplayValues.c
INT8 RestoreOldDisplayValues (void)
{
	if(NumOldGridLimit) {
		memcpy(&GridLimits, &(OldGridLimits[LastOldGridLimit]), sizeof(tOldGridLimits));

		GridLimits.x_diff = GridLimits.x_max - GridLimits.x_min;
		GridLimits.y_diff = GridLimits.y_max - GridLimits.y_min;

		if((--LastOldGridLimit) < 0) LastOldGridLimit = (MAXOLDGRIDLIMITS-1);

		NumOldGridLimit--;

		return ((INT8)TRUE);
	}

	return ((INT8)FALSE);
}


/****************************************************************************************************
* conversions between graph and pixel coordinates
****************************************************************************************************/

//-gtopx.c
GRAPHCOORDTYPE gtopx (double x)
{
	x = (((x - GridLimits.x_min) * GRID_DX) / GridLimits.x_diff);

	if (x > GRID_BXM)				/* When actual value is outside drawing range */
		x = GRID_BXM;			
	else if (x < GRID_AXM)
		x = GRID_AXM;
	
	return ((GRAPHCOORDTYPE)(x + 0.5));	/* round it and return */
}

//-gtopy.c
GRAPHCOORDTYPE gtopy (double y)
{
	y = (((GridLimits.y_max - y) * GRID_DY) / GridLimits.y_diff); 

	if (y > GRID_BYM)				/* When actual value is outside drawing range */
		y = GRID_BYM;
	else if (y < GRID_AYM)
		y = GRID_AYM;
	
	return ((GRAPHCOORDTYPE)(y + 0.5));	/* round it and return */
}


/****************************************************************************************************
* conversions between pixel and graph coordinates
****************************************************************************************************/
//-ptogx.c
double ptogx (GRAPHCOORDTYPE X, GRAPHCOORDTYPE gxpos)
{
	return ( (GridLimits.x_max * (X - gxpos) + GridLimits.x_min * (gxpos + GRID_DX - X)) / GRID_DX );
}

//-ptogy.c
double ptogy (GRAPHCOORDTYPE Y, GRAPHCOORDTYPE gypos)
{
	return ( (GridLimits.y_max * (gypos + GRID_DY - Y) + GridLimits.y_min * (Y - gypos)) / GRID_DY );
}


/****************************************************************************************************
* Calculate a good scaling
****************************************************************************************************/
//-CalcScaling.c
#ifndef MEMORYGRID

INT16 CalcScaling (double rangestart, double rangeend, double rangediff, double FAR *ScaleStart, double FAR *ScaleDiff, INT16 FAR *ScaleNum)
{
	double		start, end, diff;
	long int	temps, tempe;
	INT16			num;

	/* calc power of 10 scaling */
	diff = log10(rangediff);			/* first scale for powers of 10 */
	num = diff;							/* round it down */
	if(diff < 0.0) num--;
	diff = pow(10.0,num);				/* calc scale diff based on power of 10 */
	
	start = rangediff/diff;				/* number of scales, 1.0 - 10.0 */

	/* search good scaling */
	num = NUMSCALEFACTORS-1;
	while(start < ScaleFactors[num].Limit) num--;

	diff *= ScaleFactors[num].Factor;	/* adapt to a better number of scales */
	
	temps = rangestart/diff;			/* calc start of first line */
	if(rangestart >= 0.0) temps++;		/* round up */
	start = diff * temps;

	tempe = rangeend/diff;				/* calc end of last line */
	if(rangeend < 0.0)	tempe--;		/* round down */
	end = diff * tempe;

	num = (INT16)((tempe - temps) + 1);	/* calc number of grid lines to display */

	*ScaleStart = start;				/* return it */
	*ScaleDiff = diff;
	*ScaleNum = num;

	return num;
}

#endif /* MEMORYGRID */



/****************************************************************************************************
* Clear GraphMem
****************************************************************************************************/
//-ClearGrid.c
void ClearGrid (void)
{
/* clear the grid memory */
#ifndef MEMORYGRID

	word FAR	*GraphAdr	= GraphMem.Mem;
	INT16		temp		= GRAPHSIZE(GRID_DX+1,GRID_DY+1);

	while(temp--) *(GraphAdr++) = 0;

#else /* MEMORYGRID */

	byte FAR	*GraphAdr	= GraphMem + 4;
	INT16		temp		= GRAPHICSIZE(GRID_DX+1,GRID_DY+1) - 4;

	while(temp--) *(GraphAdr++) = 0;

#endif /* MEMORYGRID */

	/* init global variables for fresh draw */
	CrossInGridLastX = -1;
	dashon = 1;

}


/****************************************************************************************************
* Draw grid in GraphMem
****************************************************************************************************/
//-DrawGrid.c
void DrawGrid (void)
{

#ifndef MEMORYGRID

	double				start;
	GRAPHCOORDTYPE		temp;
	INT16				num;
	
	word FAR			*GraphAdr = GraphMem.Mem;

	
	/* fast clear graphmem by words */
	num = GRAPHSIZE(GRID_DX+1,GRID_DY+1);
	while(num--) *(GraphAdr++) = 0;
	
	/* Draw Rect around graph */
	LineClipped(0,0,GRID_DX,0,DRAWMODEDASH);
	LineClipped(0,0,0,GRID_DY,DRAWMODEDASH);
	LineClipped(GRID_DX,0,GRID_DX,GRID_DY,DRAWMODEDASH);
	LineClipped(0,GRID_DY,GRID_DX,GRID_DY,DRAWMODEDASH);
	
	
	/* x-grid */

	/* calc scaling */
	num = CalcScaling(GridLimits.x_min, GridLimits.x_max, GridLimits.x_diff, &(GridScaling.ScaleStartx), &(GridScaling.ScaleDiffx), &(GridScaling.ScaleNumx));

	/* draw grid */
	start = GridScaling.ScaleStartx;
	while(num--) {
		temp = gtopx(start);
		LineClipped(temp, 0, temp, GRID_DY, DRAWMODEDASH);
		start += GridScaling.ScaleDiffx;
	}
	
	
	/* y-grid */
	
	/* calc scaling */
	num = CalcScaling(GridLimits.y_min, GridLimits.y_max, GridLimits.y_diff, &(GridScaling.ScaleStarty), &(GridScaling.ScaleDiffy), &(GridScaling.ScaleNumy));
	/* draw grid */
	start = GridScaling.ScaleStarty;
	while(num--) {
		temp = gtopy(start);
		LineClipped(0, temp, GRID_DX, temp, DRAWMODEDASH);
		start += GridScaling.ScaleDiffy;
	}

#else /* MEMORYGRID */

	far_memcpy(GraphMem,GRIDMEM,GRAPHICSIZE(GRID_DX+1,GRID_DY+1));

#endif /* MEMORYGRID */

}


/****************************************************************************************************
* initialize GraphMem, allways use before all other drawing operations!!!
****************************************************************************************************/
//-InitGrid.c
void InitGrid (void)
{
#ifdef MEMORYGRID

	*((word FAR *)GraphMem)		= GRID_DX+1;
	*((word FAR *)GraphMem + 1)	= GRID_DY+1;

#else /* MEMORYGRID */

	GraphMem.dx	= GRID_DX+1;
	GraphMem.dy	= GRID_DY+1;

#endif /* MEMORYGRID */

	ClearGrid();
}


/****************************************************************************************************
* Draw Axis scaling in GraphMem
****************************************************************************************************/
//-DrawAxis.c
void DrawAxis (void)
{
	
#ifdef MEMORYGRID
	
	char c[40];

#ifdef __HITACHI__
	sprintf(c, "%.4G", GridLimits.x_min);
#else /* __HITACHI__ */
	sprintf(c, "%.5G", GridLimits.x_min);
#endif /* __HITACHI__ */
	GridPut35Str(2, 2, c, FALSE);

#ifdef __HITACHI__
	sprintf(c, "%.4G", GridLimits.x_max);
#else /* __HITACHI__ */
	sprintf(c, "%.5G", GridLimits.x_max);
#endif /* __HITACHI__ */
	GridPut35Str(GRID_DX-4*strlen(c)-2, 2, c, FALSE);

#ifdef __HITACHI__
	sprintf(c, "%.4G", GridLimits.y_min);
#else /* __HITACHI__ */
	sprintf(c, "%.5G", GridLimits.y_min);
#endif /* __HITACHI__ */
	GridPut35Str(2, GRID_DY-7, c, FALSE);

#ifdef __HITACHI__
	sprintf(c, "%.4G", GridLimits.y_max);
#else /* __HITACHI__ */
	sprintf(c, "%.5G", GridLimits.y_max);
#endif /* __HITACHI__ */
	GridPut35Str(2, 8, c, FALSE);
	
#else /* MEMORYGRID */
	
	double s;
	INT16			n;
	GRAPHCOORDTYPE	temp;
	GRAPHCOORDTYPE	xa, xe, ya, ye;
	char c[40];
	
	xa = gtopx(GridScaling.ScaleStartx);
	xe = gtopx(GridScaling.ScaleStartx + GridScaling.ScaleDiffx * (GridScaling.ScaleNumx - 1));

	ya = gtopy(GridScaling.ScaleStarty);
	ye = gtopy(GridScaling.ScaleStarty + GridScaling.ScaleDiffy * (GridScaling.ScaleNumy - 1));

	if((xa < 0) || (xe > GRID_DX) || (ya > GRID_DY) || (ye < 0)) {
			/* Grid no more to calculate due to numerical accuracy... */

#ifdef __HITACHI__
			sprintf(c, "%.4G", GridLimits.x_min);
#else /* __HITACHI__ */
			sprintf(c, "%.5G", GridLimits.x_min);
#endif /* __HITACHI__ */
			GridPut35Str(2, 2, c, FALSE);

#ifdef __HITACHI__
			sprintf(c, "%.4G", GridLimits.x_max);
#else /* __HITACHI__ */
			sprintf(c, "%.5G", GridLimits.x_max);
#endif /* __HITACHI__ */
			GridPut35Str(GRID_DX-4*strlen(c)-2, 2, c, FALSE);

#ifdef __HITACHI__
			sprintf(c, "%.4G", GridLimits.y_min);
#else /* __HITACHI__ */
			sprintf(c, "%.5G", GridLimits.y_min);
#endif /* __HITACHI__ */
			GridPut35Str(2, GRID_DY-7, c, FALSE);

#ifdef __HITACHI__
			sprintf(c, "%.4G", GridLimits.y_max);
#else /* __HITACHI__ */
			sprintf(c, "%.5G", GridLimits.y_max);
#endif /* __HITACHI__ */
			GridPut35Str(2, 8, c, FALSE);
	}
	else {
	/* x scale */
		s = GridScaling.ScaleStartx;
		n = GridScaling.ScaleNumx;
		while(n--) {
			temp = gtopx(s);
#ifdef __HITACHI__
			sprintf(c, "%.4G", s);
#else /* __HITACHI__ */
			sprintf(c, "%.5G", s);
#endif /* __HITACHI__ */
			if((GRID_DX-temp) < (4*strlen(c))) break;
			GridPut35Str(temp+2, 2, c, FALSE);
			s += GridScaling.ScaleDiffx;
		}
	
		/* y scale */
		s = GridScaling.ScaleStarty;
		n = GridScaling.ScaleNumy;
		while(n--) {
			temp = gtopy(s);
#ifdef __HITACHI__
			sprintf(c, "%.4G", s);
#else /* __HITACHI__ */
			sprintf(c, "%.5G", s);
#endif /* __HITACHI__ */
			if(temp < 13) break;
			GridPut35Str(2, temp-6, c, FALSE);
			s += GridScaling.ScaleDiffy;
		}
	}
	
#endif /* MEMORYGRID */

}


/****************************************************************************************************
* Make complete graph visible at x,y on Display
****************************************************************************************************/
//-ShowGraph.c
void ShowGraph (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
	DrawAxis();
#ifdef MEMORYGRID
	LibPutGraph(x, y, GraphMem);
#else
	LibPutGraph(x, y, (byte FAR *)(&GraphMem));
#endif
	LibPutDispBox(x, y, GRID_DX+1, GRID_DY+1);
}


/********************************************************************************
* Plot functions to draw in GraphMem											*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* Graph.c																		*
*																				*
*																				*
* Version:	V1.03																*
* Date:		09.11.2002															*
********************************************************************************/


/* exported functions */

/****************************************************************************************************
* Bresenham's line algorithm, downloaded and used from
* http://www.geocities.com/SiliconValley/Park/8209/demo5.html
* Only one change was made - translation from Pascal to C :)))
*
* -varied for performance
* -added clipping based on Cohen-Sutherland algorithm
* -added dashed draw
****************************************************************************************************/
//-LineClipped.c
void LineClipped (GRAPHCOORDTYPE x1, GRAPHCOORDTYPE y1, GRAPHCOORDTYPE x2, GRAPHCOORDTYPE y2, INT8 mode)
{
#ifdef MEMORYGRID
	byte FAR		*GraphAdr = GraphMem + 4;
#else
	byte FAR		*GraphAdr = (byte FAR *)(GraphMem.Mem);
#endif
	GRAPHCOORDTYPE	dinc1, dinc2;
	GRAPHCOORDTYPE	xinc2, yinc2;
	GRAPHCOORDTYPE	d, numpixels;
	
	
	/************************************************************************************
	* clipp line first to visible area
	* based on Cohen-Sutherland Algorithm, optimizable?!
	*************************************************************************************/

	/* calc deltas */
	GRAPHCOORDTYPE xinc1 = x2 - x1;
	GRAPHCOORDTYPE yinc1 = y2 - y1;

	dinc1 = dinc2 = 0;

	/* set binary flags to boundary exeeding events */
	if(y1 > GRID_DY)	dinc1 |= 0x01;
	else if(y1 < 0)		dinc1 |= 0x02;
	if(x1 > GRID_DX)	dinc1 |= 0x04;
	else if(x1 < 0)		dinc1 |= 0x08;
	
	if(y2 > GRID_DY)	dinc2 |= 0x01;
	else if(y2 < 0)		dinc2 |= 0x02;
	if(x2 > GRID_DX)	dinc2 |= 0x04;
	else if(x2 < 0)		dinc2 |= 0x08;

	/* correct bounds until all inside visible area */
	while(dinc1 | dinc2) {
		
		if(dinc1 & dinc2) return;				/* line out of visible area, nothing to do */
		
		if(dinc1 & 0x01)
		{
			dinc1 &= 0x0e;										/* clear bit */
			xinc1 = x2 - (x1 = x1 + xinc1 * (GRID_DY - y1) / yinc1);	/* calc new value and new differences */
			yinc1 = y2 - (y1 = GRID_DY);					
			if(x1 > GRID_DX)	dinc1 |= 0x04;						/* check bound conditions affected */
			else if(x1 < 0)		dinc1 |= 0x08;
		}
		else if(dinc1 & 0x02)
		{
			dinc1 &= 0x0d;
			xinc1 = x2 - (x1 = x1 + xinc1 * (-y1) / yinc1);
			yinc1 = y2 - (y1 = 0);
			if(x1 > GRID_DX)	dinc1 |= 0x04;
			else if(x1 < 0)		dinc1 |= 0x08;
		}
		if(dinc1 & 0x04)
		{
			dinc1 &= 0x0b;
			yinc1 = y2 - (y1 = y1 + yinc1 * (GRID_DX - x1) / xinc1);
			xinc1 = x2 - (x1 = GRID_DX);
			if(y1 > GRID_DY)	dinc1 |= 0x01;
			else if(y1 < 0)		dinc1 |= 0x02;
		}
		else if(dinc1 & 0x08)
		{
			dinc1 &= 0x07;
			yinc1 = y2 - (y1 = y1 + yinc1 * (-x1) / xinc1);
			xinc1 = x2 - (x1 = 0);
			if(y1 > GRID_DY)	dinc1 |= 0x01;
			else if(y1 < 0)		dinc1 |= 0x02;
		}
		
		if(dinc2 & 0x01)
		{
			dinc2 &= 0x0e;
			xinc1 = (x2 = x2 + xinc1 * (GRID_DY - y1) / yinc1) - x1;
			yinc1 = (y2 = GRID_DY) - y1;
			if(x2 > GRID_DX)	dinc2 |= 0x04;
			else if(x2 < 0)		dinc2 |= 0x08;
		}
		else if(dinc2 & 0x02)
		{
			dinc2 &= 0x0d;
			xinc1 = (x2 = x2 + xinc1 * (-y1) / yinc1) - x1;
			yinc1 = (y2 = 0) - y1;
			if(x2 > GRID_DX)	dinc2 |= 0x04;
			else if(x2 < 0)		dinc2 |= 0x08;
		}
		if(dinc2 & 0x04)
		{
			dinc2 &= 0x0b;
			yinc1 = (y2 = y2 + yinc1 * (GRID_DX - x1) / xinc1) - y1;
			xinc1 = (x2 = GRID_DX) - x1;
			if(y2 > GRID_DY)	dinc2 |= 0x01;
			else if(y2 < 0)		dinc2 |= 0x02;
		}
		else if(dinc2 & 0x08)
		{
			dinc2 &= 0x07;
			yinc1 = (y2 = y2 + yinc1 * (-x1) / xinc1) - y1;
			xinc1 = (x2 = 0) - x1;
			if(y2 > GRID_DY)	dinc2 |= 0x01;
			else if(y2 < 0)		dinc2 |= 0x02;
		}
	}
	
	/***********************************************
	* prepare drawing of line
	***********************************************/
	
	/* Determine deltas	*/
	xinc1 = abs(xinc1);
	yinc1 = abs(yinc1);

	/************************************************************************************************/
	/* Preparation of values for calculation of D													*/
	if (xinc1 >= yinc1)							/* Is line longer or higher? */
	{
		numpixels = xinc1;						/* x is independent variable */
		d = (yinc1 << 1) - xinc1;				/* d = 2*yinc1 - xinc1		 */
		
		dinc1 = yinc1 << 1;						/* DeltaY * 2				 */
		dinc2 = (yinc1 - xinc1) << 1;			/* (DeltaY-DeltaX) * 2		 */

		xinc1 = xinc2 = 1;						/* stepsizes to next point	 */
		yinc1 = 0;
		yinc2 = XBYTES(GRID_DX+1);
	}
	else
	{
		numpixels = yinc1;						/* y is independent variable */
		d = (xinc1 << 1) - yinc1;

		dinc1 = xinc1 << 1;
		dinc2 = (xinc1 - yinc1) << 1;

		xinc1 = 0;								/* stepsizes to next point	 */
		xinc2 = 1;
		yinc1 = yinc2 = XBYTES(GRID_DX+1);
	}
	
	/************************************************************************************************/
	/* Force steps to change x and y in correct direction											*/
	if (x1 > x2)
	{
		xinc1 = -xinc1; 
		xinc2 = -xinc2;
	}
	if (y1 > y2)
	{
		yinc1 = -yinc1; 
		yinc2 = -yinc2;
	}
	
	/************************************************************************************************/
	/* Calc Graphic memory offset																	*/
	GraphAdr += y1 * XBYTES(GRID_DX+1);

	/************************************************************************************************/
	/* at least one pixel to set																	*/
	numpixels++;
	
	/************************************************************************************************/
	/* Drawing loop	for each mode, a bit faster														*/

	switch (mode) {
	case DRAWMODESET:
		while(numpixels--)
		{
			/* Calc Memadress and set Pixel */
			*(GraphAdr + (x1>>3)) |= 0x80>>(x1 & 0x07);
			
			if (d < 0)
			{
				d  += dinc1; 
				x1 += xinc1; 
				GraphAdr += yinc1;
			}
			else
			{
				d  += dinc2; 
				x1 += xinc2; 
				GraphAdr += yinc2;
			}
		}
		break;
	case DRAWMODEDASH:
		/* invert dashstate cause new line begins at last point of last line */
		dashon ^= 1;

		while(numpixels--)
		{
			if(dashon ^= 1)
				/* Calc Memadress and set Pixel */
				*(GraphAdr + (x1>>3)) |= (0x80>>(x1 & 0x07));
			
			if (d < 0)
			{
				d  += dinc1; 
				x1 += xinc1; 
				GraphAdr += yinc1;
			}
			else
			{
				d  += dinc2; 
				x1 += xinc2; 
				GraphAdr += yinc2;
			}
		}
		break;
	case DRAWMODEDASHINVERT:
		/* invert dashstate cause new line begins at last point of last line */
		dashon ^= 1;

		while(numpixels--)
		{
			if(dashon ^= 1)
				/* Calc Memadress and set Pixel */
				*(GraphAdr + (x1>>3)) ^= (0x80>>(x1 & 0x07));
			
			if (d < 0)
			{
				d  += dinc1; 
				x1 += xinc1; 
				GraphAdr += yinc1;
			}
			else
			{
				d  += dinc2; 
				x1 += xinc2; 
				GraphAdr += yinc2;
			}
		}
		break;
	case DRAWMODEINVERT:
		while(numpixels--)
		{
			/* Calc Memadress and invert Pixel */
			*(GraphAdr + (x1>>3)) ^= (0x80>>(x1 & 0x07));
			
			if (d < 0)
			{
				d  += dinc1; 
				x1 += xinc1; 
				GraphAdr += yinc1;
			}
			else
			{
				d  += dinc2; 
				x1 += xinc2; 
				GraphAdr += yinc2;
			}
		}
		break;
	case DRAWMODEUNSET:
	default:
		while(numpixels--)
		{
			/* Calc Memadress and clear Pixel */
			*(GraphAdr + (x1>>3)) &= ~(0x80>>(x1 & 0x07));
			
			if (d < 0)
			{
				d  += dinc1; 
				x1 += xinc1; 
				GraphAdr += yinc1;
			}
			else
			{
				d  += dinc2; 
				x1 += xinc2; 
				GraphAdr += yinc2;
			}
		}
		break;
	}

}


/****************************************************************************************************
* Put number string on plot grid for scaling, with own mini font
****************************************************************************************************/
//-GridPut35Str.c
typedef struct T35CHAR {
	word	PixelBits;	/* Pixel bits from left to right and top to bottom */
	char	Character;	/* character code of character definition */
} t35Char;


t35Char FAR	CharSet35[] = {	/* my mini charset for just numbers */
	{0x0BA0, '+'},
	{0x0380, '-'},
	{0x0004, '.'},
	{0xF6DE, '0'},
	{0x2492, '1'},
	{0xE7CE, '2'},
	{0xE59E, '3'},
	{0xB792, '4'},
	{0xF39E, '5'},
	{0xF3DE, '6'},
	{0xF492, '7'},
	{0xF7DE, '8'},
	{0xF793, '9'},
	{0xF34E, 'E'},
	{0x0000,  0 }
};

void GridPut35Str (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y, char *c, INT8 vertical)
{
	byte FAR		*GraphAdr;
	t35Char FAR		*Char;
	INT16			GraphPointer;
	GRAPHCOORDTYPE	xtemp, ytemp;
	word			BitMask, CharBits;
	byte			Num, xl, SetBits;

	/* for all characters in string */
	while(*c)
	{
		/* search character in font definition */
		Char = CharSet35;
		while(Char->Character && (Char->Character != *c)) Char++;

		/* plot character */
		CharBits = Char->PixelBits;
		BitMask = 0x8000;
		Num = 3*5;
		xl = 0;

		xtemp = x;
		ytemp = y;
		GraphPointer = ytemp * XBYTES(GRID_DX+1);

		/* set/unset each pixel of character */
		if(vertical) {													/* vertical/horizontal writing? */
			while(Num--)
			{
				if(xtemp > GRID_DX) break;									/* limit to visible area */
				if((xtemp >= 0) && INGRIDRANGEY(ytemp))
				{
					/* calc byte and bit mask */
#ifdef MEMORYGRID
					GraphAdr = (GraphMem + 4) + (GraphPointer + (xtemp>>3));
#else
					GraphAdr = (byte FAR *)(GraphMem.Mem) + (GraphPointer + (xtemp>>3));
#endif
					SetBits = 0x80>>(xtemp & 0x07);

					if(CharBits & BitMask)	*GraphAdr |= SetBits;		/* set pixel */
					else					*GraphAdr &= ~SetBits;		/* clear pixel */
				}

				/* calc position of next point */
				xl++;
				if(!(xl%=3))
				{
					xtemp++;
					ytemp = y;
					GraphPointer += 2*XBYTES(GRID_DX+1);
				}
				else
				{
					ytemp--;
					GraphPointer -= XBYTES(GRID_DX+1);
				}

				BitMask>>=1;
			};
			y -= 4;														/* Position of next character */
		}
		else {
			while(Num--)
			{
				if(ytemp > GRID_DY) break;									/* limit to visible area */
				if(INGRIDRANGEX(xtemp) && (ytemp >=0))
				{
					/* calc byte and bit mask */
#ifdef MEMORYGRID
					GraphAdr = (GraphMem + 4) + (GraphPointer + (xtemp>>3));
#else
					GraphAdr = (byte FAR *)(GraphMem.Mem) + (GraphPointer + (xtemp>>3));
#endif
					SetBits = 0x80>>(xtemp & 0x07);

					if(CharBits & BitMask)	*GraphAdr |= SetBits;		/* set pixel */
					else					*GraphAdr &= ~SetBits;		/* clear pixel */
				}

				/* calc position of next point */
				xl++;
				if(!(xl%=3))
				{
					xtemp = x;
					ytemp++;
					GraphPointer += XBYTES(GRID_DX+1);
				} 
				else xtemp++;

				BitMask>>=1;
			}
			x +=4;														/* Position of next character */
		}

		c++;
	}
}


/****************************************************************************************************
* Set Dot
****************************************************************************************************/
//-SetDot.c
void SetDot (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
	if(INGRIDRANGE(x,y))
	{
		*(CalcXYMemAdress(x,y)) |= (0x80>>(x & 0x07));
	}
}


/****************************************************************************************************
* Set Dot
****************************************************************************************************/
//-ClrDot.c
void ClrDot (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
	if(INGRIDRANGE(x,y))
	{
		*(CalcXYMemAdress(x,y)) &= ~(0x80>>(x & 0x07));
	}
}


/****************************************************************************************************
* Invert Dot
****************************************************************************************************/
//-InvDot.c
void InvDot (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
	if(INGRIDRANGE(x,y))
	{
		*(CalcXYMemAdress(x,y)) ^= (0x80>>(x & 0x07));
	}
}


/****************************************************************************************************
* Get Dot State, result <0 out of range, 0 not set, 1 set
****************************************************************************************************/
//-GetDot.c
INT8 GetDot (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
	if(INGRIDRANGE(x,y))
	{
		return ((*(CalcXYMemAdress(x,y)) & (0x80>>(x & 0x07))) >> (7-(x & 0x07)));
	}

	return (-1);
}


/****************************************************************************************************
* Plot a Graph stored in Graph
****************************************************************************************************/
//-PlotGraph.c
void PlotGraph (tGraph *Graph, INT8 mode)
{
	unsigned INT16	n	= Graph->NumPoints;
	tGraphPoint		*p	= Graph->Points;
	GRAPHCOORDTYPE	x	= gtopx(p->x);
	GRAPHCOORDTYPE	y	= gtopy(p->y);

	/* reset dashstate */
	dashon = 1;

	/* plot graph in graphmem */
	while(n--) {
		LineClipped (x, y, x = gtopx(p->x), y = gtopy(p->y), mode);
		p++;
	}
}


/****************************************************************************************************
* Plot a Graph stored in xValues, yValues
****************************************************************************************************/
//-PlotGraphS.c
void PlotGraphS (double *xValues, double *yValues, unsigned INT16 NumPoints, INT8 mode)
{
	GRAPHCOORDTYPE	x	= gtopx(*xValues);
	GRAPHCOORDTYPE	y	= gtopy(*yValues);

	/* reset dashstate */
	dashon = 1;

	/* plot graph in graphmem */
	while(NumPoints--) {
		LineClipped (x, y, x = gtopx(*(xValues++)), y = gtopy(*(yValues++)), mode);
	}
}


/****************************************************************************************************
* Plot a Graph stored in Graph
****************************************************************************************************/
//-PlotGraphINT.c
void PlotGraphINT (tGraphINT *Graph, INT8 mode)
{
	unsigned INT16	n	= Graph->NumPoints;
	tGraphPointINT	*p	= Graph->Points;
	GRAPHCOORDTYPE	x	= p->x;
	GRAPHCOORDTYPE	y	= p->y;

	/* reset dashstate */
	dashon = 1;

	/* plot graph in graphmem */
	while(n--) {
		LineClipped (x, y, x = p->x, y = p->y, mode);
		p++;
	}
}


/****************************************************************************************************
* Plot a Graph stored in xValues, yValues
****************************************************************************************************/
//-PlotGraphSINT.c
void PlotGraphSINT (GRAPHCOORDTYPE *xValues, GRAPHCOORDTYPE *yValues, unsigned INT16 NumPoints, INT8 mode)
{
	GRAPHCOORDTYPE	x	= *xValues;
	GRAPHCOORDTYPE	y	= *yValues;

	/* reset dashstate */
	dashon = 1;

	/* plot graph in graphmem */
	while(NumPoints--) {
		LineClipped (x, y, x = *(xValues++), y = *(yValues++), mode);
	}
}


/****************************************************************************************************
* Draw Busy Box
****************************************************************************************************/
//-DrawBusy.c
void DrawBusy (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
	LibGdsBox(x, y, x+25, y+25);
	LibPutFarData(x+1, y+1, 24);
	LibPutDispBox(x, y, 26, 26);
}


/****************************************************************************************************
* Draw little Cross in Graphmem, remove last cross if one exists
****************************************************************************************************/
//-DrawCrossInGrid.c
void DrawCrossSingle (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
		LineClipped(x-CROSSSIZE, y, x-2, y, DRAWMODEINVERT);
		LineClipped(x, y-CROSSSIZE, x, y-2, DRAWMODEINVERT);
		LineClipped(x+2, y, x+CROSSSIZE, y, DRAWMODEINVERT);
		LineClipped(x, y+2, x, y+CROSSSIZE, DRAWMODEINVERT);
}

INT8 DrawCrossInGrid (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
	INT8 succ = 0;

	/* remove last cross if one exists */
	if(!((CrossInGridLastX < 0) || (CrossInGridLastY < 0)))
	{
		DrawCrossSingle (CrossInGridLastX, CrossInGridLastY);
		CrossInGridLastX = -1;
		succ |= 1;
	}

	/* draw new cross, if in grid range */
	if(INGRIDRANGE(x,y))
	{
		DrawCrossSingle ((CrossInGridLastX = x), (CrossInGridLastY = y));
		succ |= 1;
	}

	return succ;
}

