/*
 * modeldef.h
 *
 * common defines model dependent
 *
 * include this before all other includes
 * use it in header files, too
 *
 * you need not include define.h, libc.h or l_libc.h when
 * you use this file, it automatically includes these for you
 *
 * Thanks to Johannes Steingrber for building first version of this file
 *
 * extended by Ansgar Schfer
 *
 * V1.0
 */

#ifndef modeldef_h_included
#define modeldef_h_included


/* for a non CASIO PV compiler define NOCASIOPVCOMPILER */
#ifdef NOCASIOPVCOMPILER
/* treat a non casio compiler much like the Hitachi compiler */
#define	__HITACHI__
#endif	/* NOCASIOPVCOMPILER */


#ifdef __HITACHI__ /* Hitachi compiler */

/*
 * Hitachi compiler needs explicit inclusion of these header files
 * if they are left out wrong code for e.g. sprintf is generated
 */

#include <stdarg.h>
#include <setjmp.h>

#include <string.h>
#ifndef NOCASIOPVCOMPILER
#include <machine.h>
#endif	/* NOCASIOPVCOMPILER */

#include <math.h>
#ifndef NOCASIOPVCOMPILER
#include <mathf.h>
#endif	/* NOCASIOPVCOMPILER */

#include <float.h>
#include <limits.h>

#include <errno.h>

#include <stdlib.h>

#include <stdio.h>
#include <string.h>

/*
 * these are for the PV related things
 */
#ifndef NOCASIOPVCOMPILER
#include <hitachi.h>
#include "libc.h"
#endif	/* NOCASIOPVCOMPILER */

#ifndef NOCASIOPVCOMPILER
#define DBL_EPSILON		DBL_POS_EPS
#endif	/* NOCASIOPVCOMPILER */

/*
 * for code compatibility
 *
 * use FAR and NEAR instead of far and near in you projects
 * use far_xxxx whenever LSI compiler requires far variant
 * to be extended...
 */
#define FAR		/* nothing */
#define NEAR	/* nothing */
#define far_memcpy		memcpy
#define far_memmove		memmove
#define far_memset		memset
#define far_memcmp		memcmp
#define far_memchr		memchr
#define far_memicmp		memicmp
#define far_memccpy		memccpy

#define far_strcpy		strcpy
#define far_strncpy		strncpy
#define far_strcat		strcat
#define far_strncat		strncat
#define far_strcmp		strcmp
#define far_strncmp		strncmp
#define far_strlen		strlen
#define far_strchr		strchr
#define far_strrchr		strrchr
#define far_strstr		strstr
#define far_strspn		strspn
#define far_strcspn		strcspn
#define far_strpbrk		strpbrk
#define far_strtok		strtok
#define far_stpcpy		stpcpy
#define far_strnset		strnset
#define far_strset		strset
#define far_strrev		strrev
#define far_strlwr		strlwr
#define far_strupr		strupr
#define far_strnicmp	strnicmp
#define	far_strncmpi	strnicmp
#define far_stricmp		stricmp
#define	far_strcmpi		stricmp

#define far_strdup		strdup

#define	far_movmem(src, dst, len)	memcpy((dst), (src), (len))
#define	far_setmem(dst, len, dat)	memset((dst), (dat), (len))


/* my own integer types to get memory usage transparent */
#define INT8	char
#define INT16	short
#define INT32	int


#else /* LSI compiler for old NEC V30 processor */

/*
 * these are for the PV related things
 */

#include <stdrom.h>
#include "libc.h"
#include "l_libc.h"
#include <memory.h>
#include "L_define.h"
#include "L_libc.h"


/*
 * for code compatibility
 *
 * use FAR and NEAR instead of far and near in your projects
 * INT has different lenght depending on compiler/processor
 *
 * there is some confusion about FILE_xxxx_MODE definitions in
 * old and new SDK
 * I prefer to use the new IMB_ defines in both projects
 */

#define FAR		far
#define NEAR	near


/* my own integer types to get memory usage transparent */
#define INT8	char
#define INT16	short
#define INT32	long int

#define IMB_FILE_OPEN_MODE			FILE_OPEN_MODE
#define IMB_FILE_SECRET_MODE		FILE_SECRET_MODE

#define IML_FLASHLIB_TEXT_MAX		(2048+1)	/* Flash Interface Text Buffer Size */
#define IML_FLASHLIB_BIN_MAX		(3072)		/* Flash Interface Binary Buffer Size */

#endif /* ifdef __HITACHI__ */

#endif /* ifndef ...included */