/********************************************************************************
* table oriented Parser for mathematical formulas								*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* Parseint.h																	*
*																				*
* defines internal constants, symbols, types, and variables						*
*																				*
* Version:	V2.04																*
* Date:		01.11.2002															*
********************************************************************************/

#ifndef _PARSEINT_H
#define _PARSEINT_H

#include "modeldef.h"

#include <float.h>
#include "Parser.h"


/* Limits */
#define FUNC_NAME_MAX_LEN	5
#define EVAL_STACK_SIZE		FORMULA_MAX_LEN*3/2
#define EVAL_TREE_SIZE		FORMULA_MAX_LEN*3/2
#define PREPARSE_SIZE		FORMULA_MAX_LEN*3/2
#define TABLEEND			0
#define ZEROACCEPTVALUE		2.0*DBL_EPSILON
#define GOODDERIVDXVALUE	4.7683715820312493637383454719999e-7
#define MAXITERATIONS		200
#define STARTMAXNEWTONSTEPS	5
#define MAXHOPS				40


/* Functiontype Codes for PARSETABLE FuncType and ExpectNext */
#define NOFUNC				0x0000
#define NUMBER				0x0001
#define CONSTANT			0x0002
#define VARIABLE			0x0004
#define AFB					0x0008
#define PAFB				0x0010
#define FUNCTION			0x0020
#define PARENOPEN			0x0040
#define PARENCLOSE			0x0080
#define KOMMA				0x0100
#define	PPAFB				0x0200
#define AF					0x0400
/* Special Flags for PARSETABLE ExpectNext */
#define FPRIORITY			0x1000
#define F2PAREN				0x2000
#define FINORDER			0x4000


/* Stack Element Types */
/* typedef struct TSTACKELEMENT tStackElement; */

/* type of a formula + internal data */
/* typedef struct TFORMULAINT tFormulaInt; */

/*
typedef struct TSTACKELEMENT
{
	double				(*InterpretAs)(tStackElement FAR *, tFormulaInt FAR *);
} tStackElement;*/

typedef struct TSTACKELEMENT
{
	double				(*InterpretAs)(void FAR *, void FAR *);
} tStackElement;

/*
typedef struct TSTACKELEMENTVAL
{
	double				(*InterpretAs)(tStackElement FAR *, tFormulaInt FAR *);
	double				Value;
} tStackElementVal; */

typedef struct TSTACKELEMENTVAL
{
	double				(*InterpretAs)(void FAR *, void FAR *);
	double				Value;
} tStackElementVal;

/*
typedef struct TSTACKELEMENTFUNC
{
	double				(*InterpretAs)(tStackElement FAR *, tFormulaInt FAR *);
	double				(*Function)(void);
} tStackElementFunc; */

typedef struct TSTACKELEMENTFUNC
{
	double				(*InterpretAs)(void FAR *, void FAR *);
	double				(*Function)(void);
} tStackElementFunc;

/*
typedef struct TSTACKELEMENTFUNC1V
{
	double				(*InterpretAs)(tStackElement FAR *, tFormulaInt FAR *);
	double				(*Function)(double);
} tStackElementFunc1V; */

typedef struct TSTACKELEMENTFUNC1V
{
	double				(*InterpretAs)(void FAR *, void FAR *);
	double				(*Function)(double);
} tStackElementFunc1V;

/*
typedef struct TSTACKELEMENTFUNC2V
{
	double				(*InterpretAs)(tStackElement FAR *, tFormulaInt FAR *);
	double				(*Function)(double, double);
} tStackElementFunc2V; */

typedef struct TSTACKELEMENTFUNC2V
{
	double				(*InterpretAs)(void FAR *, void FAR *);
	double				(*Function)(double, double);
} tStackElementFunc2V;



/* Parsetable Element */
/*
typedef struct TPARSETABLEELEMENT
{
	char				FuncName[FUNC_NAME_MAX_LEN+1];
	unsigned INT8		StringLenght;
	double				(*InterpretAs)(tStackElement FAR *, tFormulaInt FAR *);
	double				(*Function)(void);
	unsigned INT16		FuncType;
	unsigned INT16		ExpectNext;
} tParsetableElement; */

typedef struct TPARSETABLEELEMENT
{
	char				FuncName[FUNC_NAME_MAX_LEN+1];
	unsigned INT8		StringLenght;
	double				(*InterpretAs)(void FAR *, void FAR *);
	double				(*Function)(void);
	unsigned INT16		FuncType;
	unsigned INT16		ExpectNext;
} tParsetableElement;



/* Preparse Element */
typedef struct TPREPARSEELEMENT
{
	tParsetableElement FAR	*Token;			/* Pointer in Parsetable */
	double					Value;			/* for Numbers */
} tPreparseElement;



/* type of a formula + internal data */
typedef struct TFORMULAINT {
	INT8					FormulaChanged;					/* Flag for change in formula, set to true, when changing formula string!!! */
	INT8					IsFormula;						/* Flag for formula in x, set to true if x is found after parsing */
	char					Formula[FORMULA_MAX_LEN+1];		/* Formula string to put your formula in */
	double					dx;								/* dx for derivative calculations, set by InitFormula to a good value */
	INT8					ParserError;					/* last error number of parser */
	char					LastErrorMsg[ERRMSG_MAX_LEN+1];	/* last error message of parser */

	/* now parser internal data, do not use or change */
	tStackElementVal FAR	*pevalstacksave;
	unsigned INT16			stacktop;
	tStackElementVal		evalstack[EVAL_STACK_SIZE];
	tStackElementVal FAR	*pevalstack;
	double					x;

	tPreparseElement		preparsemem[PREPARSE_SIZE];
	unsigned INT16			preparseelements;
	unsigned INT16			currentpreparse;
	INT16					parencounter;
	INT16					kommacounter;
	INT8					startsign;

} tFormulaInt;



/**********************
 * Internal functions *
 **********************/

extern void						IgnoreSpace (char FAR **input);
extern double					ParseNumber (char FAR **input);
extern tParsetableElement FAR	*GetNextFormulaElement (char FAR **input);
extern INT8						PreParse (tFormulaInt FAR *Formula);
extern unsigned INT16			CheckNext (tFormulaInt FAR *Formula);
extern unsigned INT16			SubParse (unsigned INT16 Expect, tFormulaInt FAR *Formula);
extern void						BuildErrorMsg (tFormulaInt FAR *Formula);
extern double					ZeroSearch (tFormula FAR *Formula, double (*func)(tFormula FAR *,double), double (*deriv)(tFormula FAR *,double), double xstart, double xend, double ZeroAccept);

#endif	/* _PARSEINT_H */