//--
/********************************************************************************
* Math Extention																*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* MathExt.c																		*
*																				*
* Math Extention routines for function evaluation								*
*																				*
* ToDo:	-Build our own mathlib													*
*																				*
*																				*
* Version:	V2.05																*
* Date:		01.11.2002															*
********************************************************************************/

#include "MathExt.h"

#include <stdio.h>
#include <string.h>


//-matherrordescr.c
/************************
 * Math Error Exception	*
 ***********************/
#ifdef NOMATHERRUSEERRNO

mexception			matherrordescr = {0,NULL,0.0,0.0,0.0};

#else /* NOMATHERRUSEERRNO */

struct exception	matherrordescr;

#endif /* NOMATHERRUSEERRNO */


/******************
 * Math Constants *
 ******************/
//-cNull.c
const double	cNull		=	NULLCONST;
//-cPi.c
const double	cPi			=	PICONST;
//-cEuler.c
const double	cEuler		=	EULERCONST;
//-cHalf.c
const double	cHalf		=	HALFCONST;
//-cFourth.c
const double	cFourth		=	FOURTHCONST;
//-cThrfour.c
const double	cThrfour	=	THRFOURCONST;




//-addv.c
/**********************************************************/
/* function: addv										  */
/* job: adds the two input values						  */
/* input: a and b										  */
/* returns: sum of a and b								  */
/**********************************************************/
double addv (double a, double b) {

	return (a+b);
}


//-subv.c
/**********************************************************/
/* function: subv										  */
/* job: subtracts value b from value a					  */
/* input: a and b										  */
/* returns: a - b										  */
/**********************************************************/
double subv (double a, double b) {

	return (a-b);
}


//-mulv.c
/**********************************************************/
/* function: mulv										  */
/* job: multiplies value a and value b					  */
/* input: a and b										  */
/* returns: a * b										  */
/**********************************************************/
double mulv (double a, double b) {

	return (a*b);
}


//-divv.c
/**********************************************************/
/* function: divv										  */
/* job: divides value a through value b					  */
/* input: a and b										  */
/* returns: a / b										  */
/**********************************************************/
double divv (double a, double b) {

	return (a/b);
}


//-pow10.c
/**********************************************************/
/* function: pow10										  */
/* job: calculates 10st power of a						  */
/* input: a												  */
/* returns: 10^a										  */
/**********************************************************/
double pow10 (double a) {

	return (pow(10.0, a));
}


//-logxy.c
/**********************************************************/
/* function: logxy										  */
/* job: calculates log a to base b						  */
/* input: a,b											  */
/* returns: log(a)/log(b)								  */
/**********************************************************/
double logxy (double a, double b) {

	return (log(a)/log(b));
}


//-round.c
/**********************************************************/
/* function: round										  */
/* job: rounds value a									  */
/* input: a												  */
/* returns: nearest integer to a						  */
/**********************************************************/
double round (double a) {

	if(fabs(a) > 0x4ffffffe) {
#ifdef NOMATHERRUSEERRNO
		errno = EDOM;
		return cNull;
#else /* NOMATHERRUSEERRNO */
		matherrordescr.type = DOMAIN;
		matherrordescr.name = "round";
		matherrordescr.arg1 = a;
		matherr(&matherrordescr);
		return cNull;
#endif /* NOMATHERRUSEERRNO */
	}

	if(a < cNull) return ((double)((long int)(a-0.5)));
	return ((double)((long int)(a + 0.5)));
}


//-rootxy.c
/**********************************************************/
/* function: rootxy										  */
/* job: calculates root of a to b						  */
/* input: a,b											  */
/* returns: a^(1/b)										  */
/**********************************************************/
double rootxy (double a, double b) {

	return (pow(a,1.0/b));
}


//-root3.c
/**********************************************************/
/* function: root3										  */
/* job: calculates third root of a						  */
/* input: a												  */
/* returns: a^(1/3)										  */
/**********************************************************/
double root3 (double a) {

	return (pow(a,1.0/3.0));
}


//-sqr.c
/**********************************************************/
/* function: sqr										  */
/* job: squares value a									  */
/* input: a												  */
/* returns: a*a											  */
/**********************************************************/
double sqr (double a) {

	return (a*a);
}


//-trunc.c
/**********************************************************/
/* function: trunc										  */
/* job: truncates value a								  */
/* input: a												  */
/* returns: integer of a closer to 0					  */
/**********************************************************/
double trunc (double a) {

	if(fabs(a) > 0x4fffffff) {
#ifdef NOMATHERRUSEERRNO
		errno = EDOM;
		return cNull;
#else /* NOMATHERRUSEERRNO */
		matherrordescr.type = DOMAIN;
		matherrordescr.name = "trunc";
		matherrordescr.arg1 = a;
		matherr(&matherrordescr);
		return cNull;
#endif /* NOMATHERRUSEERRNO */
	}

	return ((double)((long int)a));
}


//-thre.c
/**********************************************************/
/* function: thre										  */
/* job: threes value a									  */
/* input: a												  */
/* returns: a*a*a										  */
/**********************************************************/
double thre (double a) {

	return (a*a*a);
}


//-fak.c
/**********************************************************/
/* function: fak										  */
/* job: calculates fakulty								  */
/* input: n												  */
/* returns: fakulty										  */
/**********************************************************/
double fak (double n) {

	double e = 1; /* default result 1 */
	int  z = (int)n;

	if((z<0)||(z!=n)) {
#ifdef NOMATHERRUSEERRNO
		errno = EDOM;
		return cNull;
#else /* NOMATHERRUSEERRNO */
		matherrordescr.type = DOMAIN;
		matherrordescr.name = "fak";
		matherrordescr.arg1 = n;
		matherr(&matherrordescr);
		return cNull;
#endif /* NOMATHERRUSEERRNO */
	}

	if(z) {
		/* positive input */
		e = z;
		while(--z) {
			e *= z;
		}
	}

	return e;
}




/******************** 
 * Public variables * 
 ********************/
//-LastMathErrorMsg.c
char	LastMathErrorMsg[ERRMSG_MAX_LEN+1]	= "";		/* rrror String of last math error */
//-MathError.c
char	MathError							= FALSE;	/* flag for error in math functions */



//-matherr.c
/*************************** 
 * Math exceptions handler * 
 ***************************/

#ifdef NOMATHERRUSEERRNO

int	matherr (mexception *e) {

	char reason[30];

	e->type = errno;

	switch (e->type)
	{
		case EDOM:
			strcpy(reason,"argument domain");
			break;

		case EDIV:
			strcpy(reason,"argument singularity");
			break;

		case ERANGE:
			strcpy(reason,"range overflow");
			break;
	}

	if (e->type) {
		MathError = TRUE;
		sprintf(LastMathErrorMsg, "Math Error\r%s", reason);
	}

    return (1);
}

#else /* NOMATHERRUSEERRNO */

int	matherr (struct exception *e) {

	char reason[30];

	switch (e->type)
	{
		case DOMAIN:
			strcpy(reason,"argument domain");
			break;

		case SING:
			strcpy(reason,"argument singularity");
			break;

		case OVERFLOW:
			strcpy(reason,"range overflow");
			break;

		case UNDERFLOW:
			strcpy(reason,"numerical underflow");
			break;

		case TLOSS:			
			strcpy(reason,"total loss of significance");
			break;

		case PLOSS:			
			strcpy(reason,"partial loss of significance");
			break;
	}

	if (e->type) {
		MathError = TRUE;
#ifdef __HITACHI__
		sprintf(LastMathErrorMsg, "Math Error %s(%.4G):\r%s", e->name, e->arg1, reason);
#else /* __HITACHI__ */
		sprintf(LastMathErrorMsg, "Math Error %s(%.5G):\r%s", e->name, e->arg1, reason);
#endif /* __HITACHI__ */
	}

    return (1);
}

#endif /* NOMATHERRUSEERRNO */


//-BuildMathErrorMsg.c
/********************************************************
 * function: BuildMathErrorMsg							*
 * job: builds error message, resets MathError			*
 * returns: pointer to error message					*
 ********************************************************/
char *BuildMathErrorMsg (char FAR *Message) {

	if(MathError && Message) far_strcpy(Message, LastMathErrorMsg);

	MathError = FALSE;

	return LastMathErrorMsg;
}



//--
/********************************************************************************
* Parser Errors																	*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* ParseErr.c																	*
*																				*
* Parser Error numbers and Messages												*
*																				*
* Version:	V2.03																*
* Date:		01.11.2002															*
********************************************************************************/


#include "MathExt.h"


/****************** 
 * Error Messages * 
 *****************/
//-ErrorMsgs.c
const char FAR	MsgERR_OK[]				= "ok :o)";
const char FAR	MsgERR_READY[]			= "Ready :o)";
const char FAR	MsgERR_MATH[]			= "Math Error";
const char FAR	MsgERR_MEMORY[]			= "Memory Error :-(";
const char FAR	MsgERR_SYNTAX[]			= "Syntax Error :-(";
const char FAR	MsgERR_UNSUPPORTED[]	= "Unsupported or\runknown Function :(";
const char FAR	MsgERR_NOFORMULA[]		= "No Formula :o(";
const char FAR	MsgERR_EVALUATION[]		= "Internal Error :-(";
const char FAR	MsgERR_PAREN[]			= "Paren Error :-(";
const char FAR	MsgERR_KOMMA[]			= "Comma Error :-(";
const char FAR	MsgERR_BOUNDS[]			= "Bound Error :-(";
const char FAR	MsgERR_ENDLESS[]		= "Endless Search :-(";
const char FAR	MsgERR_ACCURACY[]		= "Accuracy Weak :-/";
const char FAR	MsgERR_HOPPING[]		= "Hopping around :-/";
const char FAR	MsgERR_DX[]				= "dx is zero in\rDerivative :-(";

const char FAR *ErrorMsgs[]= {
	MsgERR_OK,
	MsgERR_READY,
	MsgERR_MATH,
	MsgERR_MEMORY,
	MsgERR_SYNTAX,
	MsgERR_UNSUPPORTED,
	MsgERR_NOFORMULA,
	MsgERR_EVALUATION,
	MsgERR_PAREN,
	MsgERR_KOMMA,
	MsgERR_BOUNDS,
	MsgERR_ENDLESS,
	MsgERR_ACCURACY,
	MsgERR_HOPPING,
	MsgERR_DX
};
