//--
/********************************************************************************
* table oriented Parser for mathematical formulas								*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* ParseEvl.c																	*
*																				*
* contains stackmachine															*
* function variable is x														*
*																				*
* Version:	V2.05																*
* Date:		01.11.2002															*
********************************************************************************/


#include "ParseEvl.h"

#include <stddef.h>


//-EvaluateStackVAL.c
/****************************************************************/
/* function: EvaluateStackVAL									*/
/* job: Evaluate Stack Element									*/
/* input:	pointer to Stackelement								*/
/* returns:	calculated answer									*/
/****************************************************************/
double EvaluateStackVAL (tStackElement FAR *Element, tFormulaInt FAR *Formula) {

	return (((tStackElementVal FAR *)Element)->Value);
}


//-EvaluateStackFUNC2V.c
/****************************************************************/
/* function: EvaluateStackFUNC2V								*/
/* job: Evaluate Stack Element									*/
/* input:	pointer to Stackelement								*/
/* returns:	calculated answer									*/
/****************************************************************/
double EvaluateStackFUNC2V (tStackElement FAR *Element, tFormulaInt FAR *Formula) {

	double a, b;
	tStackElement FAR *p;
#ifdef NOMATHERRUSEERRNO
	double result;
#endif


	p = (tStackElement FAR *)(Formula->pevalstack);
	b = (p)->InterpretAs((tStackElement FAR *)((Formula->pevalstack)--),Formula);
	p = (tStackElement FAR *)(Formula->pevalstack);
	a = (p)->InterpretAs((tStackElement FAR *)((Formula->pevalstack)--),Formula);

#ifdef NOMATHERRUSEERRNO
	result = ((((tStackElementFunc2V FAR *)Element)->Function)(a,b));
	if(errno) matherr(&matherrordescr);
	return result;
#else
	return ((((tStackElementFunc2V FAR *)Element)->Function)(a,b));
#endif
}


//-EvaluateStackFUNC1V.c
/****************************************************************/
/* function: EvaluateStackFUNC1V								*/
/* job: Evaluate Stack Element									*/
/* input:	pointer to Stackelement								*/
/* returns:	calculated answer									*/
/****************************************************************/
double EvaluateStackFUNC1V (tStackElement FAR *Element, tFormulaInt FAR *Formula) {
	
	double a;
	tStackElement FAR *p;
#ifdef NOMATHERRUSEERRNO
	double result;
#endif

	p = (tStackElement FAR *)(Formula->pevalstack);
	a = (p)->InterpretAs((tStackElement FAR *)((Formula->pevalstack)--),Formula);

#ifdef NOMATHERRUSEERRNO
	result = ((((tStackElementFunc1V FAR *)Element)->Function)(a));
	if(errno) matherr(&matherrordescr);
	return result;
#else
	return ((((tStackElementFunc1V FAR *)Element)->Function)(a));
#endif
}


//-EvaluateStackX.c
/****************************************************************/
/* function: EvaluateStackX										*/
/* job: Evaluate Stack Element									*/
/* input:	pointer to Stackelement								*/
/* returns:	calculated answer									*/
/****************************************************************/
double EvaluateStackX (tStackElement FAR *Element, tFormulaInt FAR *Formula) {

	return (Formula->x);
}


//-EvaluateStackNOFUNC.c
/****************************************************************/
/* function: EvaluateStackNOFUNC								*/
/* job: Evaluate Stack Element									*/
/* input:	pointer to Stackelement								*/
/* returns:	calculated answer, builds error Message				*/
/****************************************************************/
double EvaluateStackNOFUNC (tStackElement FAR *Element, tFormulaInt FAR *Formula) {

	/* Error */
	Formula->ParserError = ERR_NOFORMULA;
	BuildErrorMsg(Formula);						/* Build an Error Message */

	return cNull;
}


//-EvaluateStackNONE.c
/****************************************************************/
/* function: EvaluateStackNONE									*/
/* job: Evaluate Stack Element									*/
/* input:	pointer to Stackelement								*/
/* returns:	calculated answer, builds error Message				*/
/****************************************************************/
double EvaluateStackNONE (tStackElement FAR *Element, tFormulaInt FAR *Formula) {

	/* Error */
	Formula->ParserError = ERR_EVALUATION;
	BuildErrorMsg(Formula);						/* Build an Error Message */

	return cNull;
}


//-EvaluateStack.c
/****************************************************************/
/* function: EvaluateStack										*/
/* job: Evaluate Stack Element									*/
/* input:	pointer to Stackelement								*/
/* returns:	calculated answer									*/
/****************************************************************/
double EvaluateStack (tStackElement FAR *Element, tFormulaInt FAR *Formula) {
#ifdef NOMATHERRUSEERRNO
	errno = 0;
#endif
	return ((Element->InterpretAs)(Element,Formula));
}


//-ClearStack.c
/******************************************************
 * function: ClearStack                               *
 * job: initializes evaluate stack                    *
 * returns: nothing                                   *
 ******************************************************/
void ClearStack (tFormulaInt FAR *Formula) {

	Formula->stacktop = EVAL_STACK_SIZE;
	Formula->pevalstack = Formula->pevalstacksave = Formula->evalstack;
	(Formula->pevalstack)->InterpretAs = (double (*)(void FAR *, void FAR *))EvaluateStackNOFUNC;
}
/* of ClearStack */


//-RestoreStack.c
/******************************************************
 * function: RestoreStack                             *
 * job: restores stack for repeated evaluation        *
 * returns: nothing                                   *
 ******************************************************/
void RestoreStack (tFormulaInt FAR *Formula) {

	Formula->pevalstack = Formula->pevalstacksave;
}
/* restores stack for repeated evaluation */


//-push.c
/******************************************************
 * function: push                                     *
 * job: pushes given element onto top of evalstack;   *
 *      if stack is full, then set ParserError flag   *
 * returns:	pointer to stackelement to fill with	  *
 *          information								  *
 *			if NULL stack is full					  *
 ******************************************************/
tStackElementVal FAR *push (tFormulaInt FAR *Formula) {

	if (Formula->stacktop) {
		Formula->stacktop--;
		Formula->pevalstacksave = Formula->pevalstack++;
		return (Formula->pevalstacksave);
	}

    Formula->ParserError = ERR_MEMORY;

	return NULL;
}
/* of push */


//-pop.c
/*************************************************
 * function: pop								 *
 * job: pops element from top of evalstack       *
 * returns: pointer to stackelemnt               *
 *************************************************/
tStackElementVal FAR *pop (tFormulaInt FAR *Formula) {

    return (Formula->pevalstack--);
}
/* of spop */



//--
/********************************************************************************
* table oriented Parser for mathematical formulas								*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* Parser.c																		*
*																				*
* contains parsing code	and stackmachine										*
* allows interpretation of formula and fast repeated evaluation					*
* function variable is x														*
*																				*
* for supported functions see ParseTbl.h										*
*																				*
* Version:	V2.03																*
* Date:		01.11.2002															*
********************************************************************************/


#include "ParseEvl.h"

#include <string.h>


/*************************************** 
 * Parser internal data and structures * 
 ***************************************/

#include "ParseTbl.h"

/******************** 
 * Public variables * 
 ********************/
//-ZeroAccept.c
double	ZeroAccept		= ZEROACCEPTVALUE;					/* Value indicating a zero, numerical accuracy... */

//-GoodDerivDX.c
double	GoodDerivDX		= GOODDERIVDXVALUE;



//-FunctionName.c
/********************************************************
 * function: FunctionName								*
 * job: delivers adress of functionname in Parsetable	*
 * returns: pointer to error message					*
 ********************************************************/
char FAR *FunctionName (INT16 index) {
	
	return ((char FAR *)ParseTable[index].FuncName);
}


//-IgnoreSpace.c
/*******************************************************
 * function: IgnoreSpace							   *
 * job: ignores spaces in input string				   *
 * returns: nothing									   *
 *          input points to character after last space *
 *******************************************************/
void IgnoreSpace (char FAR **input) {

	while(*(*input) == ' ') (*input)++;

	return;
}
/* of IgnoreSpace */


//-ParseNumber.c
/****************************************************
 * function: ParseNumber							*
 * job: Parses a number								*
 * input: input points on first character			*
 * returns:	parsed number							*
 *			input points to character after number	*
 ****************************************************/
double ParseNumber (char FAR **input) {

	double			result		= cNull;
	double			base		= 10.0;
	double			exponent;
	unsigned INT8	ExpNegFlag	= FALSE;
	unsigned INT8	NegFlag		= FALSE;

	/* sign */
	switch(*(*input)) {
		case '-':
			NegFlag = TRUE;
		case '+':
			(*input)++;
			break;
		default:
			break;
	}

	/* numbers pre . */
	while((*(*input) >= '0') && (*(*input) <= '9')) {
		result *= base;
		result += (*((*input)++) - '0');
	}

	/* . or E */
	switch(*(*input)) {
	case '.':
		/* numbers after . */
		(*input)++;
		exponent = 1.0;
		while((*(*input) >= '0') && (*(*input) <= '9')) {
			result += ((*((*input)++) - '0') * (exponent /= base));
		}
		/* E */
		switch(*(*input)) {
		case 'e':
		case 'E':
			/* sign for E */
			switch(*(++(*input))) {
			case '-':
				ExpNegFlag = TRUE;
			case '+':
				(*input)++;
				break;
			default:
				break;
			}
			/* Number for E */
			exponent = cNull;
			while((*(*input) >= '0') && (*(*input) <= '9')) {
				exponent *= base;
				exponent += (*((*input)++) - '0');
			}
			if(ExpNegFlag) exponent *= -1.0;
			result *= pow(base,exponent);
			break;
		default:
			break;
		}
		break;
	case 'e':
	case 'E':
		/* sign for E */
		switch(*(++(*input))) {
		case '-':
			ExpNegFlag = TRUE;
		case '+':
			(*input)++;
			break;
		default:
			break;
		}
		/* Number for E */
		exponent = cNull;
		while((*(*input) >= '0') && (*(*input) <= '9')) {
			exponent *= base;
			exponent += (*((*input)++) - '0');
		}
		if(ExpNegFlag) exponent *= -1.0;
		result *= pow(base,exponent);
		break;
	default:
		break;
	}

	if(NegFlag) result *= -1.0;

	return result;
}
/* of ParseNumber */


//-GetNextFormulaElement.c
/*********************************************
 * function: GetNextFormulaElement			 *
 * job: Get the next element in input string *
 * input: none								 *
 * returns: Pointer to found Type			 *
 *********************************************/
tParsetableElement FAR *GetNextFormulaElement (char FAR **input)
{
	tParsetableElement FAR *PTZ = (tParsetableElement FAR *)ParseTable;		/* Pointer in Parsetable */

	/* ignore spaces in between */
	while(*(*input) == ' ') (*input)++;

	/* find fitting function */
	while(PTZ->FuncType) {
		if(!far_strncmp(*input, PTZ->FuncName, PTZ->StringLenght)) {
			(*input) += PTZ->StringLenght;
			break;
		}
		PTZ++;
	}

	return PTZ;
}


//-PreParse.c
/***********************************************************
 * function: PreParse                                      *
 * job: converts input string in tokens found and numbers  *
 * input: none											   *
 * returns: 0 on success				                   *
 *			1 on syntax error							   *
 *			2 on memory error							   *
 ***********************************************************/
INT8 PreParse (tFormulaInt FAR *Formula) {
	char FAR				*input = Formula->Formula;
	tPreparseElement FAR	*PreparsePointer = Formula->preparsemem;

	Formula->preparseelements = 0;

	PreparsePointer->Token = NULL;

	/* search whole String */
	while(*input) {
		/* Get next syntactic element and put pointer to Parsetable in Token */
		PreparsePointer->Token = GetNextFormulaElement(&input);

		/* special handling on numbers and constants*/
		switch((PreparsePointer->Token)->FuncType) {
		case NUMBER:
			/* parse value */
			input--;
			PreparsePointer->Value = ParseNumber(&input);
			break;
		case CONSTANT:
			/* put constant in value */
			PreparsePointer->Value = *((double FAR *)((PreparsePointer->Token)->Function));
			break;
		case TABLEEND:
			Formula->ParserError = ERR_UNSUPPORTED;
			return 1;	/* no matching function string found, Error */
		default:
			break;
		}

		PreparsePointer++;
		if((++(Formula->preparseelements)) >= PREPARSE_SIZE) {
			Formula->ParserError = ERR_MEMORY;
			return 2;
		}
	}

	PreparsePointer->Token = NULL;

	return 0;
}


//-CheckNext.c
/***********************************************************
 * function: CheckNext                                     *
 * job: returns the next formula element type			   *
 * input: none											   *
 * returns: found function Element type                    *
 ***********************************************************/
unsigned INT16 CheckNext (tFormulaInt FAR *Formula) {

	tParsetableElement FAR *PTZ;		/* Pointer in Parsetable */

	/* get next Element in Formula */
	if(!(PTZ = Formula->preparsemem[Formula->currentpreparse].Token)) return 0;

	return PTZ->FuncType;
}


//-SubParse.c
/***********************************************************
 * function: SubParse                                      *
 * job: parses the next formula element	and fills Stack	   *
 *      public variable ParserError                        *
 * input: expected Element								   *
 * returns: found function Element type                    *
 ***********************************************************/
unsigned INT16 SubParse (unsigned INT16 Expect, tFormulaInt FAR *Formula) {

	tStackElementVal FAR	*Element;	/* Pointer to Element on Stack to put info in */

	tParsetableElement FAR	*PTZ;		/* Pointer in Parsetable */

	unsigned INT8			cont = 1;   /* Flag for continue Loop */

	unsigned INT16			Result;		/* Result to give back */

	unsigned INT16			SubResult;	/* Result of a Subparse */

	/* Check for a sign in a Subparse, first */

	/* get current Element in Formula */
	if(!(PTZ = Formula->preparsemem[Formula->currentpreparse].Token)) return 0;

	/* Check if + or - at the beginning, Add 0 to use it */
	if(Formula->startsign) {
		switch(PTZ->FuncType) {
		case AFB:
			if(Element = push(Formula)) {
				Element->InterpretAs = (double (*)(void FAR *, void FAR *))EvaluateStackVAL;
				Element->Value = cNull;
			}
			Expect = AFB;
			break;
		case PAFB:
		case PPAFB:
			Formula->ParserError = ERR_SYNTAX;
			return 0;
		default:
			break;
		}
	}
	Formula->startsign = FALSE;

	/* continue to Parse */

	do {
		/* get current Element in Formula */
		if(!(PTZ = Formula->preparsemem[Formula->currentpreparse].Token)) return 0;

		Result = PTZ->FuncType;

		/* check if an expected Type */
		if(!(Result & Expect)) {
			/* if not TableEnd, Error */
			if(Result) {
				Formula->ParserError = ERR_SYNTAX;
				return 0;
			}
			return PTZ->FuncType;
		}

		/* Parse on according to what found */
		switch(Result) {
		case NUMBER:
		case CONSTANT:
			/* put number on stack */
			if(Element = push(Formula)) {
				Element->InterpretAs = PTZ->InterpretAs;
				Element->Value = Formula->preparsemem[Formula->currentpreparse].Value;
			}
			Formula->currentpreparse++;
			if(Expect & FINORDER) cont = 0;
			Expect = PTZ->ExpectNext;
			break;

		case VARIABLE:
			/* put variable on stack, is a formula */
			Formula->IsFormula = TRUE;
			if(Element = push(Formula)) {
				Element->InterpretAs = PTZ->InterpretAs;
			}
			Formula->currentpreparse++;
			if(Expect & FINORDER) cont = 0;
			Expect = PTZ->ExpectNext;
			break;

		case AFB:
			/* + or - */
			Formula->currentpreparse++;
			if(!SubParse(PTZ->ExpectNext,Formula)) {
				Formula->ParserError = ERR_SYNTAX;
				return 0;
			}
			/* check next */
			while(CheckNext(Formula) & (PAFB|PPAFB|AF)) {
				Result = SubParse(PAFB|PPAFB|AF|FPRIORITY,Formula);
			}
			/* put function on stack */
			if(Element = push(Formula)) {
				Element->InterpretAs = PTZ->InterpretAs;
				((tStackElementFunc FAR *)Element)->Function = PTZ->Function;
			}
			Expect = PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB;
			break;

		case PAFB:
			/* * or / */
			Formula->currentpreparse++;
			if(!SubParse(PTZ->ExpectNext,Formula)) {
				Formula->ParserError = ERR_SYNTAX;
				return 0;
			}
			/* check next */
			do {
				switch(SubResult = CheckNext(Formula)) {
				case KOMMA:
				case PARENCLOSE:
				case AFB:
					if(Expect & FPRIORITY) cont = 0;
					break;
				case PPAFB:
				case AF:
					Result = SubParse(PPAFB|AF|FPRIORITY,Formula);
					break;
				}
			}
			while(SubResult & (PPAFB|AF));
			/* put function on stack */
			if(Element = push(Formula)) {
				Element->InterpretAs = PTZ->InterpretAs;
				((tStackElementFunc FAR *)Element)->Function = PTZ->Function;
			}
			Expect = PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB|(Expect&FPRIORITY);
			break;

		case FUNCTION:
			/* function with arguments */
			Formula->currentpreparse++;
			/* parse argument(s) */
			if(!SubParse(PTZ->ExpectNext,Formula)) {
				Formula->ParserError = ERR_SYNTAX;
				return 0;
			}
			/* put function on stack */
			if(Element = push(Formula)) {
				Element->InterpretAs = PTZ->InterpretAs;
				((tStackElementFunc FAR *)Element)->Function = PTZ->Function;
			}
			if(Expect & FINORDER) cont = 0;
			Expect = PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB;
			break;

		case PARENOPEN:
			/* treat paren as complete prior subparse */
			Formula->parencounter++;
			Formula->currentpreparse++;
			/* calculation of function with two parameters */
			if(Expect & F2PAREN) {
				/* first argument of 2 argument function */
				Formula->kommacounter++;
				Formula->startsign = TRUE;
				while((SubResult = SubParse(PTZ->ExpectNext|PAFB|PPAFB,Formula)) != KOMMA) {
					switch(SubResult) {
					case PARENCLOSE:
					case NOFUNC:
						Formula->ParserError = ERR_SYNTAX;
						return 0;
					default:
						break;
					}
				}
			}
			/* argument of function or parencalculation */
			Formula->startsign = TRUE;
			while((SubResult = SubParse(PTZ->ExpectNext|PAFB|PPAFB,Formula)) != PARENCLOSE) {
				switch(SubResult) {
				case NOFUNC:
				case KOMMA:
					Formula->ParserError = ERR_SYNTAX;
					return 0;
				default:
					break;
				}
			}
			if(Expect & FINORDER) cont = 0;
			Expect = PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB;
			break;

		case PARENCLOSE:
			/* end of subparse, stop subparse */
			Formula->parencounter--;
			Formula->currentpreparse++;
			cont = 0;
			break;

		case KOMMA:
			/* end of subparse, stop subparse */
			Formula->kommacounter--;
			Formula->currentpreparse++;
			cont = 0;
			break;

		case PPAFB:
			/* ^ */
			Formula->currentpreparse++;
			if(!SubParse(PTZ->ExpectNext,Formula)) {
				Formula->ParserError = ERR_SYNTAX;
				return 0;
			}
			/* check next */
			if((Expect & FPRIORITY) && (CheckNext(Formula) & (KOMMA|PARENCLOSE|AFB|PAFB))) cont = 0;
			/* put function on stack */
			if(Element = push(Formula)) {
				Element->InterpretAs = PTZ->InterpretAs;
				((tStackElementFunc FAR *)Element)->Function = PTZ->Function;
			}
			Expect = PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB|(Expect&FPRIORITY);
			break;

		case AF:
			/* ! */
			Formula->currentpreparse++;
			/* check next */
			if((Expect & FPRIORITY) && (CheckNext(Formula) & (KOMMA|PARENCLOSE|AFB|PAFB))) cont = 0;
			/* put function on stack */
			if(Element = push(Formula)) {
				Element->InterpretAs = PTZ->InterpretAs;
				((tStackElementFunc FAR *)Element)->Function = PTZ->Function;
			}
			Expect = PTZ->ExpectNext|(Expect&FPRIORITY);
			break;

		case TABLEEND:
			/* end of formula, stop parse */
			cont = 0;
			break;

		default:
			Formula->ParserError = ERR_SYNTAX;
			return 0;
		}
	}
	while(cont);

	/* give back what found */
	return Result;
}


//-InitFormula.c
/*********************************************************** 
 * function: InitParser                                    *
 * job: public function, that clears public variables      *
 * returns: nothing                                        *
 ***********************************************************/
void InitFormula (tFormula FAR *Formula)
{
	*(Formula->Formula)		= 0;					/* init formula string */
	Formula->FormulaChanged	= TRUE;					/* force new parse */
	Formula->ParserError	= ERR_READY;			/* reset error state */
	Formula->dx				= GoodDerivDX;			/* set a good dx */
	Formula->IsFormula		= FALSE;				/* no formula in x */
	BuildErrorMsg((tFormulaInt FAR *)Formula);		/* build an error message */

	ClearStack((tFormulaInt FAR *)Formula);			/* clear evaluation stack for new formula */
	((tFormulaInt FAR *)Formula)->x = cNull;			/* set x to zero */

	return;
}
/* of InitFormula */


//-BuildErrorMsg.c
/********************************************************
 * function: BuildErrorMsg								*
 * job: builds error message, resets MathError			*
 * returns: nothing										*
 ********************************************************/
void BuildErrorMsg (tFormulaInt FAR *Formula) {

	if(MathError) {
		Formula->ParserError = ERR_MATH;
		BuildMathErrorMsg(Formula->LastErrorMsg);
	}
	else {
		far_strcpy(Formula->LastErrorMsg, ErrorMsgs[Formula->ParserError]);
	}
}


//-Parse.c

#ifdef __HITACHI__
#ifndef NOCASIOPVCOMPILER
char *strlwr (char *c) {
	while (*c) {
		*c = (char)tolower((int)(*c));
		c++;
	}
	return c;
}
#endif /* NOCASIOPVCOMPILER */
#endif /* __HITACHI__ */

/************************************************************
 * function: Parse											*
 * job: public function, parses given string s and affects	*
 *      public variable ParserError							*
 *		parses only, if global Formula_Changed is TRUE		*
 *		else just evaluates last parsed formula				*
 * input:	pointer to Formulastring						*
 *			pointer to Change Flag							*
 *			xvalue to evaluate with							*
 * returns: parsed value									*
 *			ParserError is set according to ERROR state		*
 *			IsFormula is set to TRUE if x is found			*
 *			Formula_Changed is set to TRUE if succ. parsed	*
 ************************************************************/
double Parse (tFormula FAR *Formula, double xvalue) {

	if(((tFormulaInt FAR *)Formula)->ParserError) {
		((tFormulaInt FAR *)Formula)->ParserError = ERR_OK;		/* reset error state */
		BuildErrorMsg((tFormulaInt FAR *)Formula);				/* build an error message */
	}
	((tFormulaInt FAR *)Formula)->x = xvalue;					/* set x for evaluation */

	if(((tFormulaInt FAR *)Formula)->FormulaChanged)			/* parse only if change is signaled */
	{
		((tFormulaInt FAR *)Formula)->kommacounter = 0;
		((tFormulaInt FAR *)Formula)->parencounter = 0;

		((tFormulaInt FAR *)Formula)->IsFormula = FALSE;		/* only if x found in string, then formula... */

		far_strlwr(((tFormulaInt FAR *)Formula)->Formula);		/* all lowercase */

		if(!PreParse((tFormulaInt FAR *)Formula))				/* preparse formula, so it is easier to look ahead */
		{
			ClearStack(((tFormulaInt FAR *)Formula));			/* clear evaluation stack for new formula */
			((tFormulaInt FAR *)Formula)->currentpreparse	= 0;
			((tFormulaInt FAR *)Formula)->startsign			= TRUE;
			while(SubParse(PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION|AFB|PAFB|PPAFB,((tFormulaInt FAR *)Formula))); /* parse until end is found */
		}

		if(!(((tFormulaInt FAR *)Formula)->ParserError)) {
			/* check for paren/comma errors */
			if(((tFormulaInt FAR *)Formula)->parencounter) {
				((tFormulaInt FAR *)Formula)->ParserError = ERR_PAREN;
			}

			if(((tFormulaInt FAR *)Formula)->kommacounter) {
				((tFormulaInt FAR *)Formula)->ParserError = ERR_KOMMA;
			}
		}

		if(((tFormulaInt FAR *)Formula)->ParserError)
		{
			BuildErrorMsg((tFormulaInt FAR *)Formula);				/* Build an Error Message */
			return (cNull);										/* Error, so no evaluation */
		}

		((tFormulaInt FAR *)Formula)->FormulaChanged = FALSE;		/* don't do it again for nothing */
	}

	/* restore Evaluation Stack Pointer for new evaluation */
	/* RestoreStack(((tFormulaInt FAR *)Formula)); */
	((tFormulaInt FAR *)Formula)->pevalstack = ((tFormulaInt FAR *)Formula)->pevalstacksave;

	/* evaluate Evaluation Stack */
	/* return (EvaluateStack(pop(((tFormulaInt FAR *)Formula)),((tFormulaInt FAR *)Formula)); */
	xvalue = EvaluateStack((tStackElement FAR *)((((tFormulaInt FAR *)Formula)->pevalstack)--), (tFormulaInt FAR *)Formula);

	if(MathError)
	{
		BuildErrorMsg((tFormulaInt FAR *)Formula);					/* Build an Error Message for MathError */
	}

	return xvalue;
}


//-Derivative.c
/************************************************************
 * function: Derivative										*
 * job: public function, parses given string s and affects	*
 *      public variable ParserError							*
 *		parses only, if global Formula_Changed is TRUE		*
 *		else just evaluates last parsed formula	to 			*
 *		derivative in xvalue								*
 * input:	pointer to Formulastring						*
 *			pointer to Change Flag							*
 *			xvalue to calculate derivative in				*
 *			dx for calculating derivative with				*
 * returns: derivative in xvalue with respect to dx			*
 *			ParserError is set according to ERROR state		*
 *			IsFormula is set to TRUE if x is found			*
 *			Formula_Changed is set to TRUE if succ. parsed	*
 ************************************************************/
double	Derivative (tFormula FAR *Formula, double xvalue) {
	double yhi,ylo;

	if(Formula->dx == cNull) {
		Formula->ParserError = ERR_DX;
		BuildErrorMsg((tFormulaInt FAR *)Formula);
		return cNull;
	}
	yhi = Parse(Formula,xvalue+Formula->dx);
	if(Formula->ParserError) return cNull;
	ylo = Parse(Formula,xvalue-Formula->dx);
	if(Formula->ParserError) return cNull;

	return ((yhi - ylo)/(Formula->dx+Formula->dx));
}


//-SecDerivative.c
/************************************************************
 * function: SecDerivative									*
 * job: public function, parses given string s and affects	*
 *      public variable ParserError							*
 *		parses only, if global Formula_Changed is TRUE		*
 *		else just evaluates last parsed formula	to 			*
 *		second derivative in xvalue							*
 * input:	pointer to Formulastring						*
 *			pointer to Change Flag							*
 *			xvalue to calculate derivative in				*
 *			dx for calculating derivative with				*
 * returns: second derivative in xvalue with respect to dx	*
 *			ParserError is set according to ERROR state		*
 *			IsFormula is set to TRUE if x is found			*
 *			Formula_Changed is set to TRUE if succ. parsed	*
 ************************************************************/
double	SecDerivative (tFormula FAR *Formula, double xvalue) {
	double yhi,ylo;

	yhi = Derivative(Formula,xvalue+Formula->dx);
	if(Formula->ParserError) return cNull;
	ylo = Derivative(Formula,xvalue-Formula->dx);
	if(Formula->ParserError) return cNull;

	return ((yhi - ylo)/(Formula->dx+Formula->dx));
}


//-ZeroSearch.c
/************************************************************
 * function: ZeroSearch										*
 * job: public function, parses given string s and affects	*
 *      public variable ParserError							*
 *		parses only, if global Formula_Changed is TRUE		*
 *		else just evaluates last parsed formula				*
 * input:	pointer to Formula								*
 *			pointer to functioncalc function				*
 *			pointer to derivativecalc function				*
 *			xstart	startvalue of searchintervall			*
 *			xend	endvalue of searchintervall				*
 * returns: xvalue of zerocrossing found in formula			*
 *			ParserError is set according to ERROR state		*
 *			IsFormula is set to TRUE if x is found			*
 *			Formula_Changed is set to TRUE if succ. parsed	*
 ************************************************************/
double	ZeroSearch (tFormula FAR *Formula, double (*func)(tFormula FAR *,double), double (*deriv)(tFormula FAR *,double), double xstart, double xend, double ZeroAccept) {
	double	x, xnew, y, ynew, dy, ystart, yend, absy, absynew, xold;
	INT16	n = MAXITERATIONS;
	INT16	nnewton = STARTMAXNEWTONSTEPS;
	INT8	tl, tr;
	INT16	HopCounter = MAXHOPS;	
	
	/* swap limits if in wrong order */
	if(xend < xstart) {
		dy = xstart;
		xstart = xend;
		xend = dy;
	}
	
	/* try to find a computable start value... */
	
	/* start in the middle of interval */
	xnew = (xstart + xend)/2.0;
	/* check Formula */
	ynew = func(Formula,xnew);
	if(Formula->ParserError) {
		/* continue at interval limits on error... */
		ynew = func(Formula,xnew = xstart);
		if(Formula->ParserError) {
			ynew = func(Formula,xnew = xend);
		}
	}
	
	if(!Formula->ParserError) {
		/* we found a computable start value, now... */
		
		
		/* here limits are ok and startvalue is ok, now... */
		
		/* init variables */
		ystart	= func(Formula,xstart);
		if(Formula->ParserError) ystart	= cNull;
		yend	= func(Formula,xend);
		if(Formula->ParserError) yend	= cNull;
		x		= xnew;
		y		= absy	= absynew = ynew;
		absy	= absynew = fabs(ynew);
		MathError = FALSE;
		xold	= y/2.0;
		
		/* search zero, continue search while too bad or still getting better */
		while(((absy > ZeroAccept) || (absynew < absy)) && ((--n) >= 0)) {
			
			/* save last x and y */
			xold = x;
			y = ynew;
			absy = absynew;
			
			/* try Newton to calc new x */
			dy = deriv(Formula, (x = xnew));
			/* we can't divide by zero and don't need large shots outside current interval... */
			/* put a half interval step in between to get better limits */
			if((fabs(dy) < ZeroAccept) || !nnewton) {
#ifdef DEBUG_ZEROSEARCH
				printf("\nHalf Interval step, dy=%g",dy);
#endif /* DEBUG_ZEROSEARCH */
				/* do half interval step */
				xnew = (xstart + xend)/2.0;
				nnewton = MAXITERATIONS;
			}
			else {
#ifdef DEBUG_ZEROSEARCH
				printf("\nNewton step, dy=%g",dy);
#endif /* DEBUG_ZEROSEARCH */
				xnew = x - y/dy;	/* Newton */
				/* check overshot of interval due to small dy */
				if(xnew > xend) {
					/* try value between x and xend */
					xnew = (x + xend)/2.0;
				}
				if(xnew < xstart) {
					/* try value between xstart and x */
					xnew = (x + xstart)/2.0;
				}
				nnewton--;
			}
#ifdef DEBUG_ZEROSEARCH
			printf("  x =%-14g xn=%-14g\n",x,xnew);
#endif /* DEBUG_ZEROSEARCH */
			
			if(x == xnew) {			/* the same as before? */
				/* Try a new value in interval closer zo zero */
				dy = (xend - xstart)/4.0;
				if(fabs(yend) >= fabs(ystart))
				{
					xnew = xstart + dy;
				}
				else
				{
					xnew = xend - dy;
				}
				if(x == xnew) {			/* still the same as before? */
										/* What to do now?
					seems we are getting in trouble with numerical accuracy */
					Formula->ParserError = ERR_ACCURACY;
					BuildErrorMsg((tFormulaInt FAR *)Formula);
					break;
				}
#ifdef DEBUG_ZEROSEARCH
				printf("SameValue Helper: x =%-14g xn=%-14g\n",x,xnew);
#endif /* DEBUG_ZEROSEARCH */
			}
			else if((xnew == xold)) {
				/* we are hopping between two values */
				if(!(--HopCounter)) {
					Formula->ParserError = ERR_HOPPING;
					BuildErrorMsg((tFormulaInt FAR *)Formula);
					break;
				}
				xnew = (x + xnew)/2.0;
#ifdef DEBUG_ZEROSEARCH
				printf("HopHelper: x =%-14g xn=%-14g\n",x,xnew);
#endif /* DEBUG_ZEROSEARCH */
			}
			
			/* Search next valid xnew close to last valid x */
			do {
				/* check formula and calc ynew */
				ynew = func(Formula,xnew);
				
				/* adapt limits, so we avoid jumping around with Newton... */
				if(Formula->ParserError) {
					/* on error just adapt limit depending on position */
					if(xnew >= x) {
						xend = xnew;
						yend = cNull;
					}
					else {
						xstart = xnew;
						ystart = cNull;
					}
					/* calc a new xnew closer to x */
					xnew = (xnew + x)/2.0;
#ifdef DEBUG_ZEROSEARCH
					printf("EA: xa=%-14g ya=%-14g xe=%-14g ye=%-14g\n",xstart,ystart,xend,yend);
#endif /* DEBUG_ZEROSEARCH */
				}
				else {
					tl = (ynew*ystart < cNull);			/* y and ystart on different side of y=0 ? */
					tr = (ynew*yend < cNull);			/* y and yend on different side of y=0 ? */
					/* just one ? */
					if(tl ^ tr) {
						if(tr) {
#ifdef DEBUG_ZEROSEARCH         
							printf("left correction\n");
#endif /* DEBUG_ZEROSEARCH */
							/* solution must be between xnew and xend */
							xstart = xnew;
							ystart = ynew;
						}
						else {
#ifdef DEBUG_ZEROSEARCH         
							printf("right correction\n");
#endif /* DEBUG_ZEROSEARCH */
							/* solution must be between xstart and xnew */
							xend = xnew;
							yend = ynew;
						}
					}
					else if(tl & tr) {
						/* there must be two zeros, continue in area closer to zero */
						if(fabs(yend) >= fabs(ystart)) {
#ifdef DEBUG_ZEROSEARCH         
							printf("right correction closer zero\n");
#endif /* DEBUG_ZEROSEARCH */
							xend = xnew;
							yend = ynew;
						}
						else {
#ifdef DEBUG_ZEROSEARCH         
							printf("left correction closer zero\n");
#endif /* DEBUG_ZEROSEARCH */
							xstart = xnew;
							ystart = ynew;
						}
					}
					else {
					/*
					all on same side of y=0, what to do now?
					hopefully we are lucky with the next step...
					at least we adapt depending on an error limit
						*/
						if(ystart == cNull) { /* indicates ystart invalid */
							if(xnew < xend) {
								xend = xnew;
								yend = ynew;
							}
						}
						if(yend == cNull) { /* indicates yend invalid */
							if(xnew > xstart) {
								xstart = xnew;
								ystart = ynew;
							}
						}
					}
#ifdef DEBUG_ZEROSEARCH         
					printf("NA: xa=%-14g ya=%-14g xe=%-14g ye=%-14g\n",xstart,ystart,xend,yend);
#endif /* DEBUG_ZEROSEARCH */
				}
			}
			while(Formula->ParserError && (n--));
			
#ifdef DEBUG_ZEROSEARCH
			printf("x =%-14g y =%-14g xn=%-14g yn=%-14g\n",x,y,xnew,ynew);
			printf("xa=%-14g ya=%-14g xe=%-14g ye=%-14g\n",xstart,ystart,xend,yend);
#endif /* DEBUG_ZEROSEARCH */
			
			absynew = ynew;
			absynew = fabs(ynew);
			MathError = FALSE;

		}
			
			/* Check if it only took too long... */
		if((n < 0) && !Formula->ParserError && (absy > ZeroAccept)) {
			Formula->ParserError = ERR_ENDLESS;
			BuildErrorMsg((tFormulaInt FAR *)Formula);
		}
			
	}
	
#ifdef DEBUG_ZEROSEARCH
	printf("cycles: %i\n\n",(MAXITERATIONS - n));
#endif /* DEBUG_ZEROSEARCH */
	
	return x;
}


//-SearchxZero.c
/************************************************************
 * function: SearchxZero									*
 * job: public function, parses given string s and affects	*
 *      public variable ParserError							*
 *		parses only, if global Formula_Changed is TRUE		*
 *		else just evaluates last parsed formula				*
 * input:	pointer to Formulastring						*
 *			xstart	startvalue of searchintervall			*
 *			xend	endvalue of searchintervall				*
 * returns: xvalue of zerocrossing found					*
 *			ParserError is set according to ERROR state		*
 *			IsFormula is set to TRUE if x is found			*
 *			Formula_Changed is set to TRUE if succ. parsed	*
 ************************************************************/
double	SearchxZero (tFormula FAR *Formula, double xstart, double xend) {

	return (ZeroSearch(Formula,Parse,Derivative,xstart,xend,ZeroAccept));
}


//-SearchxZeroDeriv.c
/************************************************************
 * function: SearchxZeroDeriv								*
 * job: public function, parses given string s and affects	*
 *      public variable ParserError							*
 *		parses only, if global Formula_Changed is TRUE		*
 *		else just evaluates last parsed formula				*
 * input:	pointer to Formulastring						*
 *			xstart	startvalue of searchintervall			*
 *			xend	endvalue of searchintervall				*
 * returns: xvalue of zerocrossing found in Derivative		*
 *			ParserError is set according to ERROR state		*
 *			IsFormula is set to TRUE if x is found			*
 *			Formula_Changed is set to TRUE if succ. parsed	*
 ************************************************************/
double	SearchxZeroDeriv (tFormula FAR *Formula, double xstart, double xend) {

	return (ZeroSearch(Formula,Derivative,SecDerivative,xstart,xend,GoodDerivDX));
}



//--
/********************************************************************************
* table oriented Parser for mathematical formulas								*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* ParseTbl.c																	*
*																				*
* Parsetable for mathematical functions											*
* contains mathematical constants												*
*																				*
* Version:	V2.02																*
* Date:		01.11.2002															*
********************************************************************************/

#include "mathext.h"
#include "ParseTbl.h"

#include <stddef.h>

/***************************************************************
 * Parsetable for function strings, syntax and action on stack *
 ***************************************************************/
//-ParseTable.c
const tParsetableElement FAR ParseTable[] = {
/*		String,		StringLenght,	Evaltype,													FunctionAdress,				Functiontype,	What to expect next and special Flags */
/*  0 */{"!",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))fak,		AF,				PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/*  1 */{"(",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackNOFUNC,	(double (*)(void))NULL,		PARENOPEN,		PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION|AFB},
/*  2 */{")",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackNOFUNC,	(double (*)(void))NULL,		PARENCLOSE,		PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/*  3 */{"*",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC2V,	(double (*)(void))mulv,		PAFB,			PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION|FINORDER},
/*  4 */{"+",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC2V,	(double (*)(void))addv,		AFB,			PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION|FINORDER},
/*  5 */{",",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackNOFUNC,	(double (*)(void))NULL,		KOMMA,			PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION},
/*  6 */{"-",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC2V,	(double (*)(void))subv,		AFB,			PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION|FINORDER},
/*  7 */{".",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/*  8 */{"/",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC2V,	(double (*)(void))divv,		PAFB,			PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION|FINORDER},
/*  9 */{"0",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 10 */{"1",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 11 */{"2",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 12 */{"3",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 13 */{"4",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 14 */{"5",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 15 */{"6",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 16 */{"7",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 17 */{"8",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 18 */{"9",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))NULL,		NUMBER,			PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 19 */{"abs",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))fabs,		FUNCTION,		PARENOPEN|FINORDER},
/* 20 */{"acos",	4,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))acos,		FUNCTION,		PARENOPEN|FINORDER},
/* 21 */{"asin",	4,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))asin,		FUNCTION,		PARENOPEN|FINORDER},
/* 22 */{"atan",	4,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))atan,		FUNCTION,		PARENOPEN|FINORDER},
/* 23 */{"cosh",	4,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))cosh,		FUNCTION,		PARENOPEN|FINORDER},
/* 24 */{"cos",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))cos,		FUNCTION,		PARENOPEN|FINORDER},
/* 25 */{"exp",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))exp,		FUNCTION,		PARENOPEN|FINORDER},
/* 26 */{"e",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))&cEuler,	CONSTANT,		PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 27 */{"fak",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))fak,		FUNCTION,		PARENOPEN|FINORDER},
/* 28 */{"ln",		2,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))log,		FUNCTION,		PARENOPEN|FINORDER},
/* 29 */{"log10",	5,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))log10,	FUNCTION,		PARENOPEN|FINORDER},
/* 30 */{"log",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC2V,	(double (*)(void))logxy,	FUNCTION,		PARENOPEN|FINORDER|F2PAREN},
/* 31 */{"pi",		2,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))&cPi,		CONSTANT,		PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 32 */{"pow10",	5,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))pow10,	FUNCTION,		PARENOPEN|FINORDER},
/* 33 */{"pow",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC2V,	(double (*)(void))pow,		FUNCTION,		PARENOPEN|FINORDER|F2PAREN},
/* 34 */{"root3",	5,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))root3,	FUNCTION,		PARENOPEN|FINORDER},
/* 35 */{"root",	4,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC2V,	(double (*)(void))rootxy,	FUNCTION,		PARENOPEN|FINORDER|F2PAREN},
/* 36 */{"round",	5,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))round,	FUNCTION,		PARENOPEN|FINORDER},
/* 37 */{"sinh",	4,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))sinh,		FUNCTION,		PARENOPEN|FINORDER},
/* 38 */{"sin",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))sin,		FUNCTION,		PARENOPEN|FINORDER},
/* 39 */{"sqrt",	4,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))sqrt,		FUNCTION,		PARENOPEN|FINORDER},
/* 40 */{"sqr",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))sqr,		FUNCTION,		PARENOPEN|FINORDER},
/* 41 */{"tanh",	4,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))tanh,		FUNCTION,		PARENOPEN|FINORDER},
/* 42 */{"tan",		3,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))tan,		FUNCTION,		PARENOPEN|FINORDER},
/* 43 */{"trunc",	5,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))trunc,	FUNCTION,		PARENOPEN|FINORDER},
/* 44 */{"x",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackX,			(double (*)(void))NULL,		VARIABLE,		PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 45 */{"^",		1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC2V,	(double (*)(void))pow,		PPAFB,			PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION|FINORDER},
/* 46 */{{0xC6,0},	1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))sqr,		AF,				PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 47 */{{0xC7,0},	1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))thre,		AF,				PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 48 */{{0xC9,0},	1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))&cHalf,	CONSTANT,		PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 49 */{{0xCA,0},	1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))&cFourth,	CONSTANT,		PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 50 */{{0xCB,0},	1,				(double (*)(void FAR *, void FAR *))EvaluateStackVAL,		(double (*)(void))&cThrfour,CONSTANT,		PARENCLOSE|KOMMA|PPAFB|PAFB|AF|AFB},
/* 51 */{{0xD1,0},	1,				(double (*)(void FAR *, void FAR *))EvaluateStackFUNC1V,	(double (*)(void))sqrt,		FUNCTION,		PARENOPEN|NUMBER|CONSTANT|VARIABLE|FUNCTION|FINORDER},
/* 52 */{"",		0,				(double (*)(void FAR *, void FAR *))EvaluateStackNOFUNC,	(double (*)(void))NULL,		TABLEEND,		TABLEEND}
};
