//--
/*
  SHARED Library Functions
  V1.0

  build by somebody

  modified for Library and PV-S1600 by Ansgar Schfer
*/

#include "sharedlib.h"

#include "Libc.h"

/* Filename for SHARED File */
#define DEFSHAREDFILENAME		"SHARED"

extern char	SHAREDFILENAME[];
extern byte	SHAREDmode;
extern byte	SHAREDsubmode;

extern INT8	sffindfile(FILE_BUF *fb);
extern int	sffindrecord(char *signature,FILE_BUF *fb,FILE_INFO *fi);
extern byte	*sfgetdatapointer(FILE_BUF *fbuf,int *freebytes);


//-fi.c
FILE_INFO		SHAREDfi;						/* File Info for SHARED file	*/

//-fbub.c
FILE_BUF		SHAREDfbuf;						/* File Buffer for SHARED file	*/

//-Filename.c
char			SHAREDFILENAME[] = DEFSHAREDFILENAME;

//-Mode.c
byte			SHAREDmode=0;
byte			SHAREDsubmode=0;

//-sffindfile.c
INT8 sffindfile(FILE_BUF *fb)
{
	/* open or create shared data file */
	if (SHAREDmode==0)
	{
		if (!LibGetAllEntry((byte *)SHAREDFILENAME,&SHAREDmode,&SHAREDsubmode))	/* find shared file */
		{
			if (!LibSubEntrySave((byte *)SHAREDFILENAME,&SHAREDsubmode)) return FALSE; 
			/* create new */
			LibGetAllEntry((byte *)SHAREDFILENAME,&SHAREDmode,&SHAREDsubmode);	/* get mode and submode */
		}
    }
  /* Set main and sub entry mode in fb*/
  fb->fsb_main_entry_ = SHAREDmode;
  fb->fsb_sub_entry_  = SHAREDsubmode;
  fb->fsb_scrt_info_  = 0x80;	/* Open Mode */

  return TRUE;
}
  
//-sffindrecord.c
int sffindrecord(char *signature,FILE_BUF *fb,FILE_INFO *fi)
{
  /* search named record and load it*/
  int found=FALSE;
  fi->fp = 0xffff;             /* first search */    
  fi->kind = FILE_KIND_BIN;    /* binary file */

  LibFileFindNext(fb,fi,0);
  while ((!found) && (fi->fp!=0xffff))
    {
      if (!LibFileReadEx(fb,fi,IML_FLASHLIB_BIN_MAX/FILE_BLOCK_SIZE)) 
	return FALSE;	/* Read error */
      found = strcmp(fb->fbuf.bin.bin_buf,signature)==0; /*compare signature*/
      if (!found) LibFileFindNext(fb,fi,0);
    }
  if (!found) 
    {
      fi->fp = 0xffff;             /* prepare FILE_INFO for "write new" */  
      fi->kind = FILE_KIND_BIN;    /* binary file */
    }
  return found;
}

//-sfgetdatapointer.c
byte *sfgetdatapointer(FILE_BUF *fbuf,int *freebytes)
{
  /* pointer to real data */
  byte *ptr=fbuf->fbuf.bin.bin_buf;
  *freebytes=IML_FLASHLIB_BIN_MAX;
  while (*ptr!=0) 
    {
      ptr++;	/* skip signature */
      (*freebytes)--;
    }
  ptr++;		/* skip null byte */
  (*freebytes)--;
  return ptr;
}

//-sfLoadRecord.c
INT8 sfLoadRecord(char *signature,int size,byte FAR *buffer)
{
  int freebytes;
  byte *sptr;
  
  if (! sffindfile(&SHAREDfbuf)) return FALSE;							/* find shared file */
  if (! sffindrecord(signature,&SHAREDfbuf,&SHAREDfi))					/* find and load record */   
    return FALSE;
  
  sptr = sfgetdatapointer(&SHAREDfbuf,&freebytes);						/* get pointer to data */
  if (freebytes<size) return FALSE;										/* size too big error */

  far_memcpy (buffer, sptr, size);										/* move data to buffer */

  return TRUE;
}

//-sfSaveRecord.c
INT8 sfSaveRecord(char *signature,int size,byte FAR *buffer)
{
  byte *dptr;
  int freebytes;

  if (!sffindfile(&SHAREDfbuf)) return FALSE;							/* find or create shared file */
  sffindrecord(signature,&SHAREDfbuf,&SHAREDfi);						/* find record, but ignore error*/
 
  strcpy(SHAREDfbuf.fbuf.bin.bin_buf,signature);						/* Write signature to filebuffer*/
  dptr=sfgetdatapointer(&SHAREDfbuf,&freebytes);						/* get pointer to data */
  if (size>freebytes) return FALSE;										/* size too big error */

  far_memcpy (dptr, buffer, size);										/* move data to filebuffer */

  SHAREDfbuf.fbuf.bin.char_num = dptr-SHAREDfbuf.fbuf.bin.bin_buf+size;	/* store size in filebuffer */

  return LibFileWrite(&SHAREDfbuf,&SHAREDfi);							/* write record */
}

//-sfRemoveRecord.c
INT8 sfRemoveRecord(char *signature)
{
  if (!sffindfile(&SHAREDfbuf)) return FALSE;							/* find or create shared file */
  if (!sffindrecord(signature,&SHAREDfbuf,&SHAREDfi))					/* find record */
	return TRUE;														/* ..no need to remove */
  
  return LibFileRemove(&SHAREDfbuf,&SHAREDfi);
}
