/********************************************************************************
* Plotter Surface for drawing mathematical formulas								*
*																				*
* by Ansgar Schfer, (c)2002, original code V1.0 by Martin Rezab				*
*																				*
* PVplot.c																		*
*																				*
*																				*
* Version:	V3.07																*
* Date:		09.11.2002															*
*																				*
* ToDo:																			*
*		- get Scrollbar working, maybe completly own Editor						*
*		- build our own mathlib													*
*		- pole detection														*
*		- treeparser															*
*		- limit zoom to numerical accuracy										*
********************************************************************************/


#include "PVPlot.h"



/****************************************************************************************************
* GLOBAL variables
****************************************************************************************************/

static GRAPHCOORDTYPE	LastMarkX = -1;							/* Last Mark Grid x */
static GRAPHCOORDTYPE	LastMarkY = -1;							/* Last Mark Grid y */

static INT8				CurrentFormula;							/* index of current formula to edit/plot */
static INT8				mode;									/* Current mode */

static TCHSTS			tsts;
static TXTP				EventPrm;								/* Declaration of structure for text input */
static TCHTBL 			TchTxtBuf[3];							/* Text touch area table */
static T_SCR_POS		ComScr;									/* Scrollbarinfo */
static word				GdTbl[1];								/* Guidance comment table */
static word				sel;									/* Menu selection */

static char				PVPlotTitle[]		= TITLE;
static char				TouchToNext[]		= "Make a touch to next...";

static char				MsgSave[]			= "formulas, state and\rlimits saved";
static char				MsgRemove[]			= "data removed\rfrom SHARED file";

static char				PVPlotSharedID[]	= PVPLOTSHAREDID;	/* IDs in SHARED file */
static char				PVPlotFormulaID[]	= PVPLOTFORMULAID;
static char				PVPlotDrawFXID[]	= PVPLOTDRAWFXID;
static char				PVPlotShowID[]		= PVPLOTSHOWID;
static char				PVPlotLimitsID[]	= PVPLOTLIMITSID;

static tDispFormula		Formulas[NumFormulas+1];				/* List of formulas */

/* attention, important, keep order of these formulas, put nothing in between!!! */
static tFormula	FAR		Formula1;								/* formula 1 */
static tFormula	FAR		Formula2;								/* formula 2 */
static tFormula	FAR		Formula3;								/* formula 3 */


/****************************************************************************************************
* Debug code
****************************************************************************************************/


/****************************************************************************************************
* Handling of SHARED File Data
****************************************************************************************************/

static void BuildFormulaID(char *c, char *ItemID, int fnum)
{
	sprintf(c,"%s,%s%i", PVPlotSharedID, ItemID, fnum);
}

static void BuildLimitsID(char *c)
{
	sprintf(c,"%s,%s", PVPlotSharedID, PVPlotLimitsID);
}

static INT8 LoadState (void)
{
	tDispFormula	*F		= Formulas;
	int				fnum	= 1;
	char			c[FORMULA_MAX_LEN+1];
	tFormula FAR	*FP		= &Formula1;
	byte FAR		*Icon	= GraphIconF1;
	INT8			n		= NumFormulas;
	INT8			succ	= TRUE;

	DrawBusy(58,64);

	while(n--) {

		/* init formulas and Parser */
		F->Formula =	FP++;
		F->Show =		FALSE;
		F->Drawfx =		TRUE;
		F->Icon =		Icon;
		Icon += GRAPHICSIZE(14, 12);
		InitFormula(F->Formula);

		/* load formula and state from SHARED file */
		BuildFormulaID(c, PVPlotFormulaID, fnum);
		succ &= sfLoadRecord(c, FORMULA_MAX_LEN+1, (byte FAR *)&F->Formula->Formula);
		
		BuildFormulaID(c, PVPlotDrawFXID, fnum);
		succ &= sfLoadRecord(c, sizeof(byte), (byte FAR *)&F->Drawfx);
		
		BuildFormulaID(c, PVPlotShowID, fnum++);
		succ &= sfLoadRecord(c, sizeof(byte), (byte FAR *)&F->Show);

		F++;
	}
	/* mark last entry with NULL */
	F->Formula = NULL;

	BuildLimitsID(c);
	succ &= sfLoadRecord(c, sizeof(tGridLimits), (byte FAR *)&GridLimits);

	return succ;
}

static void SaveState (void)
{
	tDispFormula	*F		= Formulas;
	int				fnum	= 1;
	char			c[20];
	INT8			n		= NumFormulas;			
	INT8			succ	= TRUE;

	DrawBusy(58,64);

	/* save formulas and state */
	while(n--) {

		BuildFormulaID(c, PVPlotFormulaID, fnum);
		succ &= sfSaveRecord(c, FORMULA_MAX_LEN+1, (byte FAR *)F->Formula->Formula);
		
		BuildFormulaID(c, PVPlotDrawFXID, fnum);
		succ &= sfSaveRecord(c, sizeof(byte), (byte FAR *)&F->Drawfx);

		BuildFormulaID(c, PVPlotShowID, fnum++);
		succ &= sfSaveRecord(c, sizeof(byte), (byte FAR *)&F->Show);

		F++;
	}
	BuildLimitsID(c);
	succ &= sfSaveRecord(c, sizeof(tGridLimits), (byte FAR *)&GridLimits);

	if(succ) LibPutMsgDlg3((byte *)MsgSave);
}

static void RemoveState (void)
{
	int				fnum	= 1;
	char			c[20];
	INT8			n		= NumFormulas;
	INT8			succ	= TRUE;

	DrawBusy(58,64);

	while(n--) {
		
		BuildFormulaID(c, PVPlotFormulaID, fnum);
		succ &= sfRemoveRecord(c);
		
		BuildFormulaID(c, PVPlotDrawFXID, fnum);
		succ &= sfRemoveRecord(c);

		BuildFormulaID(c, PVPlotShowID, fnum++);
		succ &= sfRemoveRecord(c);
	}
	BuildLimitsID(c);
	succ &= sfRemoveRecord(c);
	
	if(succ) LibPutMsgDlg((byte *)MsgRemove);
}


/****************************************************************************************************
* Initial settings of PVPlot
****************************************************************************************************/

static void InitPVPlot (void)
{
	/* set PVPlot mode, needed for PV-S1600 */
	mode = MODE_EDIT;
	CurrentFormula	= 0;

	/* init formulas and parser and load saved state from SHARED file */
	if(!LoadState()) {
		/* set default Formula */
		far_strcpy(Formulas[0].Formula->Formula, INI_FORMULA);
		Formulas[0].Show = TRUE;
	}

	/* init Grid memory */
	InitGrid();
	/* init Display */
	LibInitDisp();
}


/****************************************************************************************************
* Draw header message
****************************************************************************************************/

static void DrawHeaderMsg (char *c)
{
	LibClrBox(0, 0, YPANEL_XPOS-1, AY);
	LibStringDsp(c, 0, 0, YPANEL_XPOS-1, IB_PFONT1);
	LibPutDispBox(0, 0, YPANEL_XPOS-1, AY);
}


/****************************************************************************************************
* Wait for a touch
****************************************************************************************************/

static unsigned INT8 WaitTouch (unsigned INT8 Remove)
{
	/************************************************************************************************/
	/* Wait for touching the screen																	*/
	LibTchStackClr();
	LibTchStackPush(NULL );
	LibTchStackPush(TchHardIcon);
	LibTchStackPush(TchAllDsp);
	LibTchInit();
	LibTchWait(&tsts);

	if(tsts.obj == OBJ_HIC_ESC) {
		if(Remove) RemoveState();
		return TRUE;
	}

	return FALSE;
}


/****************************************************************************************************
* Draw litle cross, called from DrawTrace
****************************************************************************************************/
#ifdef CROSSFORUNDEFINED
static void DrawCross(GRAPHCOORDTYPE sx, GRAPHCOORDTYPE nx, GRAPHCOORDTYPE u)
{
	nx++;
	LineClipped(sx, u-1, nx, u+1, DRAWMODESET);
	LineClipped(sx, u+1, nx, u-1, DRAWMODESET);
}
#endif /* CROSSFORUNDEFINED */

/****************************************************************************************************
* Calc new y and check for pole or jump, called from DrawTrace
****************************************************************************************************/
static INT8 CalcNewY(tCalcY *NewY, double *x4, double (*func)(tFormula FAR *,double), tFormula FAR *Formula)
{
	INT8 polestep = FALSE;

	/* test 4 points of a graph for a pole or a step */

	/* save old values */
	/* NewY->y1 = NewY->y2; */
	/* NewY->y2 = NewY->y3; */
	NewY->y3 = NewY->y4;

	NewY->dy21 = NewY->dy32;
	NewY->dy32 = NewY->dy43;

	NewY->sy = NewY->ny;

	/* NewY->err1 = NewY->err2; */
	NewY->err2 = NewY->err3;
	NewY->err3 = NewY->err4;

	/* convert to screen coordinates what we want to display */
	NewY->ny = gtopy(NewY->y3);

	/* calc new values */
	NewY->y4 = func(Formula, *x4);

	NewY->err4 = Formula->ParserError;

	NewY->dy43 = NewY->y4 - NewY->y3;

	/* check for pole */
	if(NewY->dy32 < cNull) {
		if((NewY->dy43 > cNull) && (NewY->dy21 > cNull)) polestep |= TRUE;
	}
	else
	if(NewY->dy32 > cNull) {
		if((NewY->dy43 < cNull) && (NewY->dy21 < cNull)) polestep |= TRUE;
	}

	/* check for step, no working idea yet... */
	/* if(
			(NewY->dy32 != cNull) 
			&& 
			(
				(fabs(NewY->dy43 - NewY->dy32) > fabs(NewY->dy43 - NewY->dy21))
				&&
				(fabs(NewY->dy32 - NewY->dy21) > fabs(NewY->dy43 - NewY->dy21))
			)
		) polestep |= TRUE; */
	/* if(
			(NewY->dy32 != cNull) 
			&& 
			(
				(fabs(NewY->dy43 - NewY->dy21) <= (GRID_DX)*DBL_EPSILON)
				&&
				(fabs(NewY->dy32 - NewY->dy21) >= (GRID_DX)*DBL_EPSILON)
			)
		) polestep |= TRUE; */

	return polestep;
}

/****************************************************************************************************
* Draw x-trace in drawing memory, called from Draw
****************************************************************************************************/
static void DrawTrace (tDispFormula *F)
{
	tFormula FAR	*Formula	= F->Formula;
	double			*x			= GridxValuesExt;
	tCalcY			y			= {0};
	tCalcY			dy			= {0};
	GRAPHCOORDTYPE	n			= GRID_DX;
	GRAPHCOORDTYPE	sx			= 0;
	GRAPHCOORDTYPE	nx			= 0;
#ifdef CROSSFORUNDEFINED
	GRAPHCOORDTYPE	u = gtopy(cNull);
#endif /* CROSSFORUNDEFINED */
	INT8			DrawMode	= DRAWMODEDASH;
	INT8			polestep;

	
	if((!(*(Formula->Formula))) || (!F->Show)) return;			/* don't draw if formula is empty */

	/* set drawing mode */
	if(F->Drawfx) DrawMode = DRAWMODESET;

	/* init calculation and pole detection */
	CalcNewY(&y,x,&Parse,Formula);
	if(DrawMode) CalcNewY(&dy,x,&Derivative,Formula);
	CalcNewY(&y,++x,&Parse,Formula);
	if(DrawMode) CalcNewY(&dy,x,&Derivative,Formula);
	CalcNewY(&y,++x,&Parse,Formula);
	if(DrawMode) CalcNewY(&dy,x,&Derivative,Formula);


	/* Draw trace */
	while(n--) 
	{	
		nx++;

		polestep = CalcNewY(&y,++x,&Parse,Formula);
			
		if (!(y.err3 | y.err2))
		{
			if(!polestep) LineClipped(sx, y.sy, nx, y.ny, DrawMode);
		}
#ifdef CROSSFORUNDEFINED
		else
		{
			/* litle cross for undefined value */
			DrawCross(sx, nx, u);
		}
#endif /* CROSSFORUNDEFINED */

		if(DrawMode) {

			polestep = CalcNewY(&dy,x,&Derivative,Formula);

			if (!(dy.err3 | dy.err2))
			{
				if(!polestep) LineClipped(sx, dy.sy, nx, dy.ny, DRAWMODESET);
			}
#ifdef CROSSFORUNDEFINED
			else
			{
				/* litle cross for undefined value */
				DrawCross(sx, nx, u);
			}
#endif /* CROSSFORUNDEFINED */
		}
			
		sx = nx;
	}
}


/****************************************************************************************************
* Draw complete graph
****************************************************************************************************/

static void Draw (void)
{
	tDispFormula *F = Formulas;

	DrawBusy(58,64);
	/************************************************************************************************/
	/* Clear drawing mem and draw grid																*/
	DrawGrid();
	/************************************************************************************************/
	/* calculate all x-values of the grid															*/
	CalcGridxValuesExt(&GridLimits);
	/************************************************************************************************/
	/* Draw all Formulas																			*/
	while(F->Formula) {
		DrawTrace(F++);
	}
	/************************************************************************************************/
	/* Make it visible																				*/
	ShowGraph(AX,AY);
	LastMarkX = -1;

	DrawHeaderMsg("Touch the graph area...");
}


/****************************************************************************************************
* Mark a dot in a graph
****************************************************************************************************/
static void PlotDotMark (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y)
{
	GRAPHCOORDTYPE xa, xe, ya, ye;

	if(INGRIDRANGE(x,y))
	{
		xa = x-MARKGRAPHSIZE;
		if(xa < 0) xa = 0;
		xe = x+MARKGRAPHSIZE;
		if(xe > GRID_DX) xe = GRID_DX;
		ya = y-MARKGRAPHSIZE;
		if(ya < 0) ya = 0;
		ye = y+MARKGRAPHSIZE;
		if(ye > GRID_DY) ye = GRID_DY;

		xe = xe-xa;
		ye = ye-ya;
		xa += AX;
		ya += AY;

		LibReverse(xa,ya,++xe,++ye);
		LibPutDispBox(xa,ya,xe,ye);
	}
}

static void MarkGraphDot (double x, double y, INT8 removeonly)
{
	GRAPHCOORDTYPE gx = gtopx(x);
	GRAPHCOORDTYPE gy = gtopy(y);


	/* remove last mark if one exists */
	PlotDotMark(LastMarkX, LastMarkY);
	LastMarkX = -1;

	/* draw new mark, if in grid range */
	if(!removeonly) PlotDotMark((LastMarkX=gx), (LastMarkY=gy));

}


/****************************************************************************************************
* Draw Zero Crossing
****************************************************************************************************/
static void PlotMode_TchInit(void);
static unsigned INT8 DrawZeroCrossing (INT8 pformula)
{
#ifdef EXTREMAANALYSIS
	int				fnum	= pformula;
#endif
	unsigned INT8	redraw	= FALSE;
	double			x, d;
#ifdef RESCALEONZEROCROSSING
	double			dy;
#endif
	double			y;
	char			c[60];
	char			ci[5];
	INT8			Error = FALSE;

	fnum++;


	DrawBusy(58,64);

	LastMarkX = -1;

	if (Formulas[pformula].Drawfx) {
		sprintf(ci,"f%i", fnum);
		x = SearchxZero(Formulas[pformula].Formula,GridLimits.x_min,GridLimits.x_max);
	}
	else {
		sprintf(ci,"f%i'", fnum);
		x = SearchxZeroDeriv(Formulas[pformula].Formula,GridLimits.x_min,GridLimits.x_max);
	}

	if(Error = Formulas[pformula].Formula->ParserError) far_strcpy(c,Formulas[pformula].Formula->LastErrorMsg);

	switch (Error)
	{
	case ERR_MATH:
	case ERR_EVALUATION:
	case ERR_BOUNDS:
	case ERR_ENDLESS:
	case ERR_DX:
		ShowGraph(AX,AY);
		break;
	case ERR_ACCURACY:
	case ERR_HOPPING:
		LibPutMsgDlg((byte *)c);
#ifdef __HITACHI__
		sprintf(c,"%s check at x:%.4G", ci, x);
#else /* __HITACHI__ */
		sprintf(c,"%s check at x:%.5G", ci, x);
#endif /* __HITACHI__ */
		ShowGraph(AX,AY);
		if (Formulas[pformula].Drawfx) {
			y = Parse(Formulas[pformula].Formula,x);
		}
		else {
			y = Derivative(Formulas[pformula].Formula,x);
		}
		MarkGraphDot(x, y, FALSE);		/* highlight the point in some way */
		break;
	default:
#ifdef EXTREMAANALYSIS
		if (!(Formulas[pformula].Drawfx))
		{
			DrawHeaderMsg (c);

			/************************************************************************************************/
			/* Draw something on the screen																	*/
			LibPutFarData(16, 16, 106);
			sprintf(c,"f%i'(x) zero at", fnum);
			LibStringDsp(c, 22, 20, 159, IB_PFONT3);

			sprintf(c,"x =");
			LibStringDsp(c, 22, 43, 159, IB_PFONT1);
#ifdef __HITACHI__
			sprintf(c,"%.7G",x);
#else /* __HITACHI__ */
			sprintf(c,"%.8G",x);
#endif /* __HITACHI__ */
			LibStringDsp(c, 60, 43, 159, IB_PFONT1);

			y = Parse(Formulas[pformula].Formula,x);

			sprintf(c,"f%i(x) =", fnum);
			LibStringDsp(c, 22, 57, 159, IB_PFONT1);
#ifdef __HITACHI__
			sprintf(c,"%.7G", y);
#else /* __HITACHI__ */
			sprintf(c,"%.8G", y);
#endif /* __HITACHI__ */
			LibStringDsp(c, 60, 57, 159, IB_PFONT1);

			y = Derivative(Formulas[pformula].Formula,x);

			sprintf(c,"f%i'(x) :", fnum);
			LibStringDsp(c, 22, 71, 159, IB_PFONT1);
#ifdef __HITACHI__
			sprintf(c,"%.5G", y);
#else /* __HITACHI__ */
			sprintf(c,"%.6G", y);
#endif /* __HITACHI__ */
			LibStringDsp(c, 60, 71, 159, IB_PFONT1);

			d = SecDerivative(Formulas[pformula].Formula,x);

			sprintf(c,"f%i''(x) :", fnum);
			LibStringDsp(c, 22, 83, 159, IB_PFONT1);
#ifdef __HITACHI__
			sprintf(c,"%.5G", d);
#else /* __HITACHI__ */
			sprintf(c,"%.6G", d);
#endif /* __HITACHI__ */
			LibStringDsp(c, 60, 83, 159, IB_PFONT1);

			if(d > ZeroAccept)			sprintf(c,"assumed minimum");
			else if(d < -ZeroAccept)	sprintf(c,"assumed maximum");
			else						sprintf(c,"saddle point???");
			LibStringDsp(c, 22, 98, 159, IB_PFONT1);

			LibStringDsp(TouchToNext, 24, 130, 159, IB_PFONT1);
			LibPutDisp();

			WaitTouch(FALSE);

			PlotMode_TchInit();

		}
#endif /* EXTREMAANALYSIS */

#ifdef __HITACHI__
		sprintf(c,"%s zero at x:%.4G", ci, x);
#else /* __HITACHI__ */
		sprintf(c,"%s zero at x:%.5G", ci, x);
#endif /* __HITACHI__ */

#ifdef RESCALEONZEROCROSSING

		/* rescale graph */
		d = GridLimits.x_diff/2.0;
		dy = GridLimits.y_diff/2.0;

		redraw = SetDisplayValues(x-d, x+d, -dy, dy);
		
#else /* RESCALEONZEROCROSSING */

		ShowGraph(AX,AY);
		MarkGraphDot(x, y, FALSE);		/* highlight the point in some way */

#endif /* RESCALEONZEROCROSSING */
		break;
	}
	
	/****************************************************/
	/* show result										*/
	DrawHeaderMsg (c);

	return redraw;
}


/****************************************************************************************************
* Draw coordinates X:#### y:##### as header
****************************************************************************************************/

static void DrawCoords (INT8 pformula, GRAPHCOORDTYPE xp)
{
	int		fnum = pformula;
	double	x, ans;
	char	c[60];
	char	ci[5];
	INT8	Error;

	fnum++;

												/****************************************************/
	LibClrBox(0, 0, YPANEL_XPOS-1, AY);			/* Clear previous									*/
	
	x = ptogx(xp,AX);							/****************************************************/
#ifdef __HITACHI__
	sprintf(c,"x:%.4G",x);						/* draw x:#######									*/
#else /* __HITACHI__ */
	sprintf(c,"x:%.5G",x);						/* draw x:#######									*/
#endif /* __HITACHI__ */
	LibStringDsp(c, 0, 0, YPANEL_XPOS/2-4, IB_PFONT1);
	
												/****************************************************/
	if (Formulas[pformula].Drawfx)				/* drawfx = 0 - drawing f'(x):#######				*/
	{											/* drawfx = 1 - drawing f(x):#######				*/
		ans = Parse(Formulas[pformula].Formula,x);
		sprintf(ci,"f%i", fnum);
	}
	else
	{
		ans = Derivative(Formulas[pformula].Formula,x);
		sprintf(ci,"f%i'", fnum);
	}

	if(Error = Formulas[pformula].Formula->ParserError)	sprintf(c,"%s: not defined", ci);
#ifdef __HITACHI__
	else sprintf(c,"%s:%.4G", ci, ans);
#else /* __HITACHI__ */
	else sprintf(c,"%s:%.5G", ci, ans);
#endif /* __HITACHI__ */


	MarkGraphDot(x, ans, Error);		/* highlight the point in some way */

	LibStringDsp(c, YPANEL_XPOS/2-3, 0, YPANEL_XPOS-1, IB_PFONT1);

	LibPutDispBox(0, 0, YPANEL_XPOS-1, AY);
}


/****************************************************************************************************
* Draw button for formula
****************************************************************************************************/

static void DrawFuncButton(void)
{
	LibPutGraph(FUNC_CHANGE_XPOS, FUNC_CHANGE_YPOS, Formulas[CurrentFormula].Icon);
	LibPutDispBox(FUNC_CHANGE_XPOS, FUNC_CHANGE_YPOS, 14, 12);
}


/****************************************************************************************************
* Draw switch to next formula
****************************************************************************************************/

static void SwitchFunc(char increment)
{
	INT8 n = 0;

	do {
		if(!(Formulas[++CurrentFormula].Formula)) {
			CurrentFormula = 0;
			if(n++) {				/* search only one time */
				n = 0;
				do {				/* no show? now search a non empty formula */
					if(!(Formulas[++CurrentFormula].Formula)) {
						CurrentFormula = 0;
						if(n++) break;		/* search only one time */
					}
				}
				while(!(*(Formulas[CurrentFormula].Formula->Formula)));
				Formulas[CurrentFormula].Show = TRUE;
			break;
			}
		}
	}
	while((!increment) && ((!(Formulas[CurrentFormula].Show)) || (!(*(Formulas[CurrentFormula].Formula->Formula)))));
}




/*****************************************************************************************************
*****************************************************************************************************
*****************************************************************************************************
****         **    *******       ***          **    ****    ***       ***        ***         ********
*****  *****  **  *******  *****  ** ***  *** ***    **    ***  *****  ***  ****  ***  ***** ********
*****  *****  **  *******  *****  ******  *******  *    *  ***  *****  ***  ****  ***  **************
*****        ***  *******  *****  ******  *******  **  **  ***  *****  ***  ****  ***      **********
*****  *********  *******  *****  ******  *******  ******  ***  *****  ***  ****  ***  **************
*****  *********  *******  *****  ******  *******  ******  ***  *****  ***  ****  ***  **************
*****  *********  **** **  *****  ******  *******  ******  ***  *****  ***  ****  ***  ***** ********
****    *******        ***       ******    *****    ****    ***       ***        ***         ********
*****************************************************************************************************
*****************************************************************************************************
*****************************************************************************************************/

static void PlotMode_Draw (void)
{
	if(!Formulas[CurrentFormula].Show) SwitchFunc(FALSE);

	LibClrDisp();
	LibPutGraph(XPANEL_XPOS, XPANEL_YPOS, GraphXPanel);
	LibPutGraph(YPANEL_XPOS, YPANEL_YPOS, GraphYPanel);
	Draw();
	DrawFuncButton();
    LibPutDisp();
}


static void PlotMode_TchInit (void)
{
	LibTchStackClr();
	LibTchStackPush(NULL);
	LibTchStackPush(TchHardIcon);
	LibTchStackPush(TchXPanelList);
	LibTchStackPush(TchYPanelList);
	LibTchStackPush(TchPlotModeList);
	LibTchInit();
}

static void PutDisplRectArea (GRAPHCOORDTYPE xa, GRAPHCOORDTYPE ya, GRAPHCOORDTYPE xe, GRAPHCOORDTYPE ye)
{
	GRAPHCOORDTYPE tmp, dx, dy;

	if(xa > xe) {
		tmp = xa;
		xa = xe;
		xe = tmp;
	}
	if(ya > ye) {
		tmp = ya;
		ya = ye;
		ye = tmp;
	}
	dx = xe-xa+1;
	dy = ye-ya+1;

	LibGdsBoxCmp(xa,ya,xe,ye);
	LibPutDispBox(xa,ya,dx,1);
	LibPutDispBox(xa,ya,1,dy);
	LibPutDispBox(xa,ye,dx,1);
	LibPutDispBox(xe,ya,1,dy);
}

static INT8 PlotMode_Run (void) 
{
	unsigned INT8	redraw = FALSE;
	unsigned INT8	Paper = FALSE;
	double			tmp, x_min_copy, x_max_copy, y_min_copy, y_max_copy;
	GRAPHCOORDTYPE	txa, tya, txe, tye, tdx, tdy;
	char			c[ERRMSG_MAX_LEN+1];
	
	PlotMode_Draw();
	PlotMode_TchInit();
	
	while (1)
	{
		LibTchWait(&tsts);

		switch(tsts.obj)
		{
		case OBJ_PAPER:
			if(tsts.act & ACT_MAKE)
			{
				Paper = TRUE;
				txa = txe = tsts.x;
				tya = tye = tsts.y;
				PutDisplRectArea(txa,tya,txe,tye);
				DrawCoords(CurrentFormula, tsts.x);
			}
			if(tsts.act & (ACT_MOVE|ACT_DOWN_IN))
			{
				PutDisplRectArea(txa,tya,txe,tye);
				txe = tsts.x;
				tye = tsts.y;
				PutDisplRectArea(txa,tya,txe,tye);
				DrawCoords(CurrentFormula, tsts.x);
			}
			if(tsts.act & ACT_BREAK_IN)
			{
				PutDisplRectArea(txa,tya,txe,tye);
				txe = tsts.x;
				tye = tsts.y;
				PutDisplRectArea(txa,tya,txe,tye);
				DrawCoords(CurrentFormula, tsts.x);
				Paper = FALSE;
			}
			if(tsts.act & (ACT_BREAK|ACT_BREAK_IN))
			{
				PutDisplRectArea(txa,tya,txe,tye);
				tdx = txe-txa;
				tdy = tye-tya;
				if((abs(tdx) > 2) && (abs(tdy) > 2))
				{
					x_min_copy = ptogx(txa,AX);
					x_max_copy = ptogx(txe,AX);
					y_min_copy = ptogy(tya,AY);
					y_max_copy = ptogy(tye,AY);
					if(tdx < 0) {
						tmp = x_min_copy;
						x_min_copy = x_max_copy;
						x_max_copy = tmp;
					}
					if(tdy > 0) {
						tmp = y_min_copy;
						y_min_copy = y_max_copy;
						y_max_copy = tmp;
					}
					
					redraw = SetDisplayValues(x_min_copy,
						x_max_copy,
						y_min_copy,
						y_max_copy);
				}
				Paper = FALSE;
			}
			break;
			
		case OBJ_FAST_LEFT:
			if (LibIconClick(&TFastLeftIcon, &tsts))
			{
				redraw = SetDisplayValues(GridLimits.x_min-GridLimits.x_diff,
					GridLimits.x_max-GridLimits.x_diff,
					GridLimits.y_min,
					GridLimits.y_max);
			}
			break;
			
		case OBJ_LEFT:
			if (LibIconClick(&TLeftIcon, &tsts))
			{
				tmp = GridLimits.x_diff/x_div;

				redraw = SetDisplayValues(GridLimits.x_min-tmp,
					GridLimits.x_max-tmp,
					GridLimits.y_min,
					GridLimits.y_max);
			}
			break;
			
		case OBJ_H_ZOOM_IN:
			if (LibIconClick(&THZoomInIcon, &tsts))
			{
				tmp = GridLimits.x_diff/4;
				
				redraw = SetDisplayValues(GridLimits.x_min+tmp,
					GridLimits.x_max-tmp,
					GridLimits.y_min,
					GridLimits.y_max);
			}
			break;
			
		case OBJ_H_CENTER:
			if (LibIconClick(&THCenterIcon, &tsts))
			{
				tmp = GridLimits.x_diff/2;

				redraw = SetDisplayValues(-tmp,
					tmp,
					GridLimits.y_min,
					GridLimits.y_max);
			}
			break;
			
		case OBJ_H_ZOOM_OUT:
			if (LibIconClick(&THZoomOutIcon, &tsts))
			{
				tmp = GridLimits.x_diff/2;

				redraw = SetDisplayValues(GridLimits.x_min-tmp,
					GridLimits.x_max+tmp,
					GridLimits.y_min,
					GridLimits.y_max);
			}
			break;
			
		case OBJ_RIGHT:
			if (LibIconClick(&TRightIcon, &tsts))
			{
				tmp = GridLimits.x_diff/x_div;

				redraw = SetDisplayValues(GridLimits.x_min+tmp,
					GridLimits.x_max+tmp,
					GridLimits.y_min,
					GridLimits.y_max);
			}
			break;
			
		case OBJ_FAST_RIGHT:
			if (LibIconClick(&TFastRightIcon, &tsts))
			{
				redraw = SetDisplayValues(GridLimits.x_min+GridLimits.x_diff,
					GridLimits.x_max+GridLimits.x_diff,
					GridLimits.y_min,
					GridLimits.y_max);
			}
			break;
			
		case OBJ_FAST_UP:
			if (LibIconClick(&TFastUpIcon, &tsts))
			{
				redraw = SetDisplayValues(GridLimits.x_min,
					GridLimits.x_max,
					GridLimits.y_min+GridLimits.y_diff,
					GridLimits.y_max+GridLimits.y_diff);
			}
			break;
			
		case OBJ_UP:
			if (LibIconClick(&TUpIcon, &tsts))
			{
				tmp = GridLimits.y_diff/y_div;

				redraw = SetDisplayValues(GridLimits.x_min,
					GridLimits.x_max,
					GridLimits.y_min+tmp,
					GridLimits.y_max+tmp);
			}
			break;
			
		case OBJ_V_ZOOM_IN:
			if (LibIconClick(&TVZoomInIcon, &tsts))
			{
				tmp = GridLimits.y_diff/4;
				
				redraw = SetDisplayValues(GridLimits.x_min,
					GridLimits.x_max,
					GridLimits.y_min+tmp,
					GridLimits.y_max-tmp);
			}
			break;
			
		case OBJ_V_CENTER:
			if (LibIconClick(&TVCenterIcon, &tsts))
			{
				tmp = GridLimits.y_diff/2;

				redraw = SetDisplayValues(GridLimits.x_min,
					GridLimits.x_max,
					-tmp,
					tmp);
			}
			break;
			
		case OBJ_V_ZOOM_OUT:
			if (LibIconClick(&TVZoomOutIcon, &tsts))
			{
				tmp = GridLimits.y_diff/2;

				redraw = SetDisplayValues(GridLimits.x_min,
					GridLimits.x_max,
					GridLimits.y_min-tmp,
					GridLimits.y_max+tmp);
			}
			break;
			
		case OBJ_DOWN:
			if (LibIconClick(&TDownIcon, &tsts))
			{
				tmp = GridLimits.y_diff/y_div;

				redraw = SetDisplayValues(GridLimits.x_min,
					GridLimits.x_max,
					GridLimits.y_min-tmp,
					GridLimits.y_max-tmp);
			}
			break;
			
		case OBJ_FAST_DOWN:
			if (LibIconClick(&TFastDownIcon, &tsts))
			{
				redraw = SetDisplayValues(GridLimits.x_min,
					GridLimits.x_max,
					GridLimits.y_min-GridLimits.y_diff,
					GridLimits.y_max-GridLimits.y_diff);
			}
			break;
			
		case OBJ_UNDO:
			if (LibIconClick(&TUndoIcon, &tsts))
			{
				redraw = RestoreOldDisplayValues();
			}
			break;

		case OBJ_G_ZOOM_IN:
			if (LibIconClick(&TGZoomInIcon, &tsts))
			{
				x_min_copy = GridLimits.x_diff/4;
				y_min_copy = GridLimits.y_diff/4;
				
				redraw = SetDisplayValues(GridLimits.x_min + x_min_copy,
					GridLimits.x_max - x_min_copy,
					GridLimits.y_min + y_min_copy,
					GridLimits.y_max - y_min_copy);
			}
			break;
			
		case OBJ_G_ZOOM_OUT:
			if (LibIconClick(&TGZoomOutIcon, &tsts))
			{
				x_min_copy = GridLimits.x_diff/2;
				y_min_copy = GridLimits.y_diff/2;

				redraw = SetDisplayValues(GridLimits.x_min - x_min_copy,
					GridLimits.x_max + x_min_copy,
					GridLimits.y_min - y_min_copy,
					GridLimits.y_max + y_min_copy);
			}
			break;
			
		case OBJ_HEADER:
			if (Formulas[CurrentFormula].Formula->ParserError)
			{
				far_strcpy(c,Formulas[CurrentFormula].Formula->LastErrorMsg);
				LibPutMsgDlg3(c);
			}
			break;
			
		case OBJ_S_ZERO:
			if (LibIconClick(&TSZeroIcon, &tsts))
			{
				redraw = DrawZeroCrossing(CurrentFormula);
			}
			break;
			
		case OBJ_FUNC_CHANGE:
			if (LibIconClick(&TIconFuncChng, &tsts))
			{
				SwitchFunc(FALSE);
				DrawFuncButton();
			}
			break;

		case OBJ_SET:
			if (LibIconClick(&TSetIcon, &tsts))
			{
				return (MODE_SETUP);
			}
			break;
			
		case OBJ_EDIT:
			if (LibIconClick(&TEditIcon, &tsts))
			{
				return (MODE_EDIT);
			}
			break;
			
		case OBJ_HIC_ESC:
			return (MODE_EXIT);
		}

		if((tsts.act & ACT_BREAK) && Paper)
		{
			PutDisplRectArea(txa,tya,txe,tye);
			Paper = FALSE;
		}

		if(redraw) {
			Draw();
			redraw = FALSE;
		}
	}
	
}


/****************************************************************************************************
* Shows info and about screens, called form EditMode_Run()
****************************************************************************************************/
static void InfoScreen (void)
{
	tInfoLine *h = Help;

	/* show all help windows */
	while(h->ypos != ENDOFHELP) {
		LibPutFarData(16, 16, 106);
		LibStringDsp(PVPlotTitle, 22,  20, 159, IB_PFONT3);
		/* draw all text of a help window */
		while(h->ypos >= 0) {
			LibStringDsp(h->text, 22,  h->ypos, 159, h->font);
			h++;
		}
		if(h->ypos == ENDOFWINDOW)	LibStringDsp(TouchToNext,				24, 130, 159, IB_PFONT1);
		else						LibStringDsp("Make a touch to quit...",	24, 130, 159, IB_PFONT1);
		/* display it */
		LibPutDisp();						 
		/* Wait for touching the screen	*/
		if(h->ypos == ENDOFWINDOW) {
			if (WaitTouch(FALSE)) return;
			h++;
		}
		else WaitTouch(TRUE);
	}
}


/****************************************************************************************************
*****************************************************************************************************
*****************************************************************************************************
****         **        ****    ***          **    ****    ***       ***        ***         **********
*****  ***** ***  ****  ****  **** ***  *** ***    **    ***  *****  ***  ****  ***  ***** **********
*****  *********  ****  ****  ********  *******  *    *  ***  *****  ***  ****  ***  ****************
*****      *****  ****  ****  ********  *******  **  **  ***  *****  ***  ****  ***      ************
*****  *********  ****  ****  ********  *******  ******  ***  *****  ***  ****  ***  ****************
*****  *********  ****  ****  ********  *******  ******  ***  *****  ***  ****  ***  ****************
*****  ***** ***  ****  ****  ********  *******  ******  ***  *****  ***  ****  ***  ***** **********
****         **        ****    ******    *****    ****    ***       ***        ***         **********
*****************************************************************************************************
*****************************************************************************************************
*****************************************************************************************************/

static void EditMode_Draw (void)
{
	LibClrDisp();												/* Clear screen										*/
	
	LibStringDsp(PVPlotTitle, 1, 1, 159, IB_PFONT3);			/* Draw title										*/
	LibGdsBox(0, 14, 159, 87);									/* Draw rectangle around the formula				*/

	LibPutGraph(OWNKEYB_XPOS, OWNKEYB_YPOS, GraphOwnKeyboard);	/* Draw Keys */

	LibPutGraph(EDITMODE_INFO_XPOS, EDITMODE_INFO_YPOS, GraphIconInfo);	/* Draw pushbutton	?								*/

	LibPutGraph(EDITMODE_CLR_XPOS, EDITMODE_CLR_YPOS, GraphIconClr);

	LibPutGraph(EDITMODE_PLOT_XPOS, EDITMODE_PLOT_YPOS, GraphIconPlot);

	DrawFuncButton();
	
    LibPutDisp();												/* Make drawing visible								*/
}


static void EditMode_TchInit (void)
{
	LibTchStackClr();
	LibTchStackPush(NULL);
	LibTchStackPush(TchHardIcon);
	LibTchStackPush(TchEditModeList);
	LibTchInit();
}


static INT8 EditMode_Run (void)
{
	
	unsigned INT8	changed = FALSE;

	char			buf[FORMULA_MAX_LEN+1];

	double			ans;

	int				FuncIndex;
	char FAR		*FuncName;
	char			KeyCode;
	char			c[ERRMSG_MAX_LEN+1];


	far_strcpy(buf, Formulas[CurrentFormula].Formula->Formula);

	/****************************************************************************************************/
	/**/GdTbl[0]		= 0xffff;			/* No Guidance comment										*/
	/**/EventPrm.st_x	= 6;				/* Start coordinate (X) of text display						*/
	/**/EventPrm.st_y	= 20;				/* Start coordinate (Y) of text display						*/
	/**/EventPrm.ed_x	= 154;				/* End coordinate (X) of text display						*/
	/**/EventPrm.it_y	= 9;				/* Text display line spacing (Y)							*/
	/**/EventPrm.MAXGYO	= 7;				/* Number of text display lines								*/
	/**/EventPrm.font	= IB_PFONT1;		/* Display font type										*/
	/**/EventPrm.csen	= TRUE;				/* Cursor display enabled(Keyword registration enabled)		*/
	/**/EventPrm.rtnen	= 0x02;				/* CR code display enabled									*/
	/**/EventPrm.maxmj	= FORMULA_MAX_LEN;	/* Maximum number of allowable input characters.			*/
	/**/EventPrm.txbf	= (byte *)buf;		/* Designation of text buffer address						*/
	/**/EventPrm.gdcmt	= GdTbl;			/* Guidance comment table									*/
	/**/EventPrm.txtobj	= OBJ_STRING;		/* Object when text area is touched.						*/
	/**/EventPrm.sbrobj	= NULL;				/* Object when the scroll bar is touched.					*/
	/**/EventPrm.tchtb	= TchTxtBuf;		/* Pointer for text scroll bar area							*/
	/**/EventPrm.s_pos	= &ComScr;			/* Pointer for text and scroll bar information				*/
	/****************************************************************************************************/
	
	EditMode_Draw();					/* Draw mode												*/

	EditMode_TchInit();					/* Initialization of touch icons							*/
	
	LibTxtInit(&EventPrm);				/* Initialization of text input	Display						*/
	
	LibTxtTchSet(&EventPrm);			/* Text touch area PUSH										*/
	
	LibTchStackPush(TchOwnKeyboardList);

	
	while (1)
	{
		if(LibTxtDsp(&EventPrm)) {
			LibPutGraph(OWNKEYB_XPOS, OWNKEYB_YPOS, GraphOwnKeyboard);	/* Draw EditKeys */
			LibPutDisp();
		}
		
		KeyCode = KEY_NONE;
		
		LibTchWait(&tsts);
		
		switch (tsts.obj)
		{
		case OBJ_DELETE:
			if (LibIconClick(&TIconDelete, &tsts))
			{
				KeyCode = KEY_DEL;
				changed = TRUE;
			}
			break;
			
		case OBJ_HIC_MBAR:
			
			LibPullDownInit(EditMenuIndexArray, SystMenuIndexArray, OptsMenuIndexArray);
			sel = LibPullDown();
			
			if (sel & PULLDOWN_EDIT)
			{
				sel &= ~PULLDOWN_IDX_MASK;
				switch (sel)
				{
				case 0x0001:
					EventPrm.txtst = TXTCUT;
					Formulas[CurrentFormula].Formula->FormulaChanged = TRUE;
					changed = TRUE;
					break;
					
				case 0x0002:
					EventPrm.txtst = TXTCOPY;
					break;
					
				case 0x0004:
					EventPrm.txtst = TXTPASTE;
					Formulas[CurrentFormula].Formula->FormulaChanged = TRUE;
					changed = TRUE;
					break;
				}
			}
			break;
			
		case OBJ_CLR:
			if (LibIconClick(&TIconClr, &tsts))
			{
				*buf = 0;
				LibTxtInit(&EventPrm);
				changed = TRUE;
			}
			break;
			
		case OBJ_PLOT:
			if (LibIconClick(&TIconPlot, &tsts))
			{
				far_strcpy(Formulas[CurrentFormula].Formula->Formula, buf);
				Formulas[CurrentFormula].Formula->FormulaChanged = TRUE;

				ans = Parse(Formulas[CurrentFormula].Formula,cNull);
				
				switch (Formulas[CurrentFormula].Formula->ParserError)
				{
				case ERR_OK:
				case ERR_MATH:
					if (Formulas[CurrentFormula].Formula->IsFormula) {				/* Expression syntax OK, x found	*/ 
						if(changed) Formulas[CurrentFormula].Show = changed;
						LibCurBlnkOff();
						return (MODE_PLOT);
					}
					else
					{
						if(Formulas[CurrentFormula].Formula->ParserError) {			/* any Error left?					*/
							far_strcpy(c,Formulas[CurrentFormula].Formula->LastErrorMsg);
							LibPutMsgDlg3(c);
						}
						else
						{
#ifdef __HITACHI__
							sprintf(c, "result for f%i:\r\r %.13G", CurrentFormula+1, ans);
#else /* __HITACHI__ */
							sprintf(c, "result for f%i:\r\r %.14G", CurrentFormula+1, ans);
#endif /* __HITACHI__ */
							LibPutMsgDlg(c);			/* put result on the display		*/
						}
					}
					break;
				default:								/* a serious error occured */
					far_strcpy(c,Formulas[CurrentFormula].Formula->LastErrorMsg);
					LibPutMsgDlg3(c);
					break;
				}
			}
			break;
			
		case OBJ_INFO:
			if (LibIconClick(&TIconInfo, &tsts))
			{
				LibCurBlnkOff();
				/****************************************************************************/
				/* Call of the InfoScreen() will kill touch info							*/
				InfoScreen();
				/****************************************************************************/
				/* Return to the editor - redraw mode and renew touch info					*/
				EditMode_Draw();
				EditMode_TchInit();
				/****************************************************************************/
				/* Reset software keyboard													*/
				LibTxtInit(&EventPrm);
				LibTxtTchSet(&EventPrm);
				
				/*LibPutGraph(0, KOFFS, EDITKEYS); */	/* Draw EditKeys */
				LibTchStackPush(TchOwnKeyboardList);
			}
			break;
			
		case OBJ_FUNC_CHANGE:
			if (LibIconClick(&TIconFuncChng, &tsts))
			{
				/* check formula */
				far_strcpy(Formulas[CurrentFormula].Formula->Formula, buf);
				Formulas[CurrentFormula].Formula->FormulaChanged = TRUE;

				ans = Parse(Formulas[CurrentFormula].Formula,cNull);

				switch (Formulas[CurrentFormula].Formula->ParserError)
				{
				case ERR_NOFORMULA:
					Formulas[CurrentFormula].Show = FALSE;
				case ERR_OK:
				case ERR_MATH:
					if(changed) Formulas[CurrentFormula].Show = changed;
					changed = FALSE;
					/* switch to next formula */
					SwitchFunc(TRUE);
					DrawFuncButton();
					far_strcpy(buf, Formulas[CurrentFormula].Formula->Formula);
					LibTxtInit(&EventPrm);
					break;
				default:	/* error in formula, has to be corrected */
					far_strcpy(c,Formulas[CurrentFormula].Formula->LastErrorMsg);
					LibPutMsgDlg3(c);
					break;
				}
			}
			break;

		case OBJ_HIC_ESC:
			LibCurBlnkOff();
			return (MODE_EXIT);
			
		default:
			if(OWNKEYB_INRANGE(tsts.obj))
			{
				FuncIndex = OWNKEYB_OBJTOOFFS(tsts.obj);
				if(LibIconClick(&TIconOwnKeyboard[FuncIndex], &tsts))
				{
					changed = TRUE;
					KeyCode = *(FuncName = FunctionName(FuncIndex));
					
					while(*(++FuncName)) {
						LibTxtInp(KeyCode, &tsts, &EventPrm);
						LibTxtDsp(&EventPrm);
						KeyCode = *FuncName;
					}
				}
			}
			break;
		}

		LibTxtInp(KeyCode, &tsts, &EventPrm);
	}
}


/****************************************************************************************************
* Editor for floating point numbers, called from SetupMode_Run()
****************************************************************************************************/
static void EditNum_TchInit (GRAPHCOORDTYPE ypos)
{
	LibTchStackClr();
	LibTchStackPush(NULL );
	LibTchStackPush(TchHardIcon);
	LibTchStackPush(TchEdNumList);

	if(ypos != SETUPMODE_NUM_YPOS+0*SETUPMODE_NUM_YSPACE+1) LibTchStackPush(TchEditXMINModeList);
	if(ypos != SETUPMODE_NUM_YPOS+1*SETUPMODE_NUM_YSPACE+1) LibTchStackPush(TchEditXMAXModeList);
	if(ypos != SETUPMODE_NUM_YPOS+2*SETUPMODE_NUM_YSPACE+1) LibTchStackPush(TchEditYMINModeList);
	if(ypos != SETUPMODE_NUM_YPOS+3*SETUPMODE_NUM_YSPACE+1) LibTchStackPush(TchEditYMAXModeList);

	LibTchInit();
}

static unsigned INT8 EditNum (double *f, GRAPHCOORDTYPE ypos)
{
	tFormula	Numbuf;
	double		value;

	INT16		FuncIndex;
	char FAR	*FuncName;
	char		KeyCode;
	INT8		ok;

	
	InitFormula(&Numbuf);

#ifdef __HITACHI__
	sprintf(Numbuf.Formula, "%.5G", *f);		/************************************************************/
#else /* __HITACHI__ */
	sprintf(Numbuf.Formula, "%.6G", *f);		/************************************************************/
#endif /* __HITACHI__ */
	GdTbl[0]		= 0xffff;					/* No Guidance comment										*/
	EventPrm.st_x	= SETUPMODE_NUM_XPOS+35;	/* Start coordinate (X) of text display						*/
	EventPrm.st_y	= ypos;						/* Start coordinate (Y) of text display						*/
	EventPrm.ed_x	= 148;						/* End coordinate (X) of text display						*/
	EventPrm.it_y	= 9;						/* Text display line spacing (Y)							*/
	EventPrm.MAXGYO	= 1;						/* Number of text display lines								*/
	EventPrm.font	= IB_PFONT1;				/* Display font type										*/
	EventPrm.csen	= TRUE;						/* Cursor display enabled(Keyword registration enabled)		*/
	EventPrm.rtnen	= 0x02;						/* CR code display enabled									*/
	EventPrm.maxmj	= FORMULA_MAX_LEN/3;		/* Maximum number of allowable input characters.			*/
	EventPrm.txbf	= (byte *)(Numbuf.Formula);	/* Designation of text buffer address						*/
	EventPrm.gdcmt	= GdTbl;					/* Guidance comment table									*/
	EventPrm.txtobj	= OBJ_STRING;				/* Object when text area is touched.						*/
	EventPrm.sbrobj	= NULL;						/* Object when the scroll bar is touched.					*/
	EventPrm.tchtb	= TchTxtBuf;				/* Pointer for text scroll bar area							*/
	EventPrm.s_pos	= &ComScr;					/* Pointer for text and scroll bar information				*/

	LibClrBox(SETUPMODE_SAVE_XPOS, SETUPMODE_SAVE_YPOS, 26, 12);
	
	LibLine(0, 87, 160, 1, 1);
	LibBox(SETUPMODE_NUM_XPOS+35-3, ypos-3, 98, 13, 1);
	
	LibKeyInit();
	SYS_KEY_FLG=SKEY_COM1_L;
	LibPutGraph(OWNKEYB_XPOS, OWNKEYB_YPOS, GraphOwnKeyboard);	/* Draw EditKeys */
	
	LibPutDisp();						/* Make drawing visible										*/
	
	EditNum_TchInit(ypos);

	LibTxtInit(&EventPrm);				/* Initialization of text input								*/
	LibTxtTchSet(&EventPrm);			/* Text touch area PUSH										*/
	
	LibTchStackPush(TchOwnKeyboardListNumEdit);
	
	while (1)
	{
		if(LibTxtDsp(&EventPrm)) {
			LibPutGraph(OWNKEYB_XPOS, OWNKEYB_YPOS, GraphOwnKeyboard);	/* Draw EditKeys */
			LibPutDisp();
		}
		
		KeyCode = KEY_NONE;
		ok = FALSE;
		
		LibTchWait(&tsts);
		
		switch(tsts.obj)
		{
		case OBJ_DELETE:
			if (LibIconClick(&TIconDelete, &tsts))
				KeyCode = KEY_DEL;
			break;
			
		case OBJ_CLR:	
			if (LibIconClick( &TClrIcon, &tsts)) 
			{ 
				*(Numbuf.Formula) = 0; 
				LibTxtInit(&EventPrm); 
			} 
			break;
											/************************************************/
		case OBJ_HIC_MBAR:					/* Same as in EditMode_Run()					*/
			LibPullDownInit(EditMenuIndexArray, SystMenuIndexArray, OptsMenuIndexArray);
			sel = LibPullDown();
			if (sel & PULLDOWN_EDIT)
			{
				sel &= ~PULLDOWN_IDX_MASK;
				switch (sel)
				{
				case 0x0001:
					EventPrm.txtst = TXTCUT;
					break;
					
				case 0x0002:
					EventPrm.txtst = TXTCOPY;
					break;
					
				case 0x0004:
					EventPrm.txtst = TXTPASTE;
					break;
				}
			}
			break;
			
		case OBJ_XMIN:
			if(LibIconClick(&TXMinIcon, &tsts)) ok = 0x02;
		case OBJ_XMAX:
			if(LibIconClick(&TXMaxIcon, &tsts)) ok = 0x04;
		case OBJ_YMIN:
			if(LibIconClick(&TYMinIcon, &tsts)) ok = 0x08;
		case OBJ_YMAX:
			if(LibIconClick(&TYMaxIcon, &tsts)) ok = 0x10;
		case OBJ_OK:
			if(LibIconClick( &TOKIcon, &tsts))	ok = 0x01; 
			if (ok) 
			{
				LibCurBlnkOff();
				value = Parse(&Numbuf,cNull);
				if(Numbuf.ParserError) {
					LibPutMsgDlg3(Numbuf.LastErrorMsg);
				}
				else {
					*f = value;
					return ok;
				}
			}
			break;
			
		case OBJ_OWNESC:	
			if (LibIconClick( &TEscIcon, &tsts))
			{
				LibCurBlnkOff();
				return TRUE;
			}
			break;
			
		case OBJ_HIC_ESC:
			LibCurBlnkOff();
			return TRUE;
			
		default:
			if(OWNKEYB_INRANGE(tsts.obj))
			{
				FuncIndex = OWNKEYB_OBJTOOFFS(tsts.obj);
				if(LibIconClick(&TIconOwnKeyboard[FuncIndex], &tsts))
				{
					KeyCode = *(FuncName = FunctionName(FuncIndex));
					
					while(*(++FuncName)) {
						LibTxtInp(KeyCode, &tsts, &EventPrm);
						LibTxtDsp(&EventPrm);
						KeyCode = *FuncName;
					}
				}
			}
			break;
		}
		LibTxtInp(KeyCode,&tsts,&EventPrm);
	}

	return TRUE;
}


/*****************************************************************************************************
*****************************************************************************************************
*****************************************************************************************************
**      ***         **        **    **    **        ***    ****    ***       ***        ***         *
*  ****  ***  ***** ** **  ** ***  ****  ****  ****  ***    **    ***  *****  ***  ****  ***  ***** *
*  *********  ***********  ******  ****  ****  ****  ***  *    *  ***  *****  ***  ****  ***  *******
**      ****      *******  ******  ****  ****       ****  **  **  ***  *****  ***  ****  ***      ***
*******  ***  ***********  ******  ****  ****  *********  ******  ***  *****  ***  ****  ***  *******
*  ****  ***  ***********  ******  ****  ****  *********  ******  ***  *****  ***  ****  ***  *******
*  ****  ***  ***** *****  ******  ****  ****  *********  ******  ***  *****  ***  ****  ***  ***** *
**      ***         ****    ******      ****    *******    ****    ***       ***        ***         *
*****************************************************************************************************
*****************************************************************************************************
*****************************************************************************************************/

static void SetupMode_TchInit (void)
{
	LibTchStackClr();
	LibTchStackPush(NULL );
	LibTchStackPush(TchHardIcon);
	LibTchStackPush(TchSetupModeList);
	LibTchStackPush(TchEdNumList);
	LibTchInit();
}


static void SetupMode_Draw_Button (GRAPHCOORDTYPE xpos, GRAPHCOORDTYPE ypos, char *name, double value)
{
	char c[40];

	LibPutGraph(SETUPMODE_NUM_XPOS, ypos, GraphIconEmptyKey);					/* button					*/
	LibPutProStr(IB_PFONT1, SETUPMODE_NUM_XPOS+xpos, ypos+1, name, 30);	
#ifdef __HITACHI__
	sprintf(c, "%.5G", value);	
#else /* __HITACHI__ */
	sprintf(c, "%.6G", value);	
#endif /* __HITACHI__ */
	LibPutProStr(IB_PFONT1, SETUPMODE_NUM_XPOS+35, ypos+1, c, 80);
}


static void SetupMode_Draw_FormulaModes (int numf, tDispFormula *F)
{
	char c[40];

	sprintf(c, "f%u", numf);	
	LibPutProStr(IB_PFONT1, SETUPMODE_FM_XPOS, SETUPMODE_FM_YPOS+numf*SETUPMODE_FM_YSPACE, c, 80);

	/* [x] check box show */
	LibPutFarData(SETUPMODE_FSHOW_XPOS, SETUPMODE_FM_YPOS+numf*SETUPMODE_FM_YSPACE, (F->Show)?17:16);
	/* [x] check box f'(x) */
	LibPutFarData(SETUPMODE_FDX_XPOS, SETUPMODE_FM_YPOS+numf*SETUPMODE_FM_YSPACE, (F->Drawfx)?16:17);
}


static void SetupMode_Draw (double x_min_copy, double x_max_copy, double y_min_copy, double y_max_copy, tDispFormula *F_copy)
{
	INT8 n = 0;

	LibClrDisp();
	
	LibStringDsp("Setup", 0, 0, 159, IB_PFONT3);
	LibBox(0, 14, 160, 160-14, 1);
	
	/* [xmin] button					*/
	SetupMode_Draw_Button(4, SETUPMODE_NUM_YPOS+0*SETUPMODE_NUM_YSPACE, "xmin", x_min_copy);
	
	/* [xmax] button					*/
	SetupMode_Draw_Button(2, SETUPMODE_NUM_YPOS+1*SETUPMODE_NUM_YSPACE, "xmax", x_max_copy);
	
	/* [ymin] button					*/
	SetupMode_Draw_Button(4, SETUPMODE_NUM_YPOS+2*SETUPMODE_NUM_YSPACE, "ymin", y_min_copy);

	/* [ymax] button					*/
	SetupMode_Draw_Button(3, SETUPMODE_NUM_YPOS+3*SETUPMODE_NUM_YSPACE, "ymax", y_max_copy);

	LibLine(0, 87, 160, 1, 1);

	/* checkboxes for formulas */
	LibStringDsp("Formula", SETUPMODE_FM_XPOS-14, SETUPMODE_FM_YPOS, 159, IB_PFONT1);
	LibStringDsp("Show", SETUPMODE_FSHOW_XPOS-8, SETUPMODE_FM_YPOS, 159, IB_PFONT1);
	LibStringDsp("f'(x)", SETUPMODE_FDX_XPOS-7, SETUPMODE_FM_YPOS, 159, IB_PFONT1);
	while (F_copy->Formula) {
		SetupMode_Draw_FormulaModes (++n, F_copy++);
	}

	/* buttons */
	LibPutGraph(SETUPMODE_SAVE_XPOS, SETUPMODE_SAVE_YPOS, GraphIconSave);
	LibPutGraph(SETUPMODE_OK_XPOS, SETUPMODE_OK_YPOS, GraphIconOK);
	LibPutGraph(SETUPMODE_ESC_XPOS, SETUPMODE_ESC_YPOS, GraphIconEsc);
	LibPutGraph(SETUPMODE_CLR_XPOS, SETUPMODE_CLR_YPOS, GraphIconClr);

	/* Update screen */
	LibPutDisp();

}


static INT8 SetupMode_Run (void)
{
	/* to work on copies only until it's ok */
	double			x_min_copy = GridLimits.x_min;
	double			x_max_copy = GridLimits.x_max;
	double			y_min_copy = GridLimits.y_min;
	double			y_max_copy = GridLimits.y_max;
	unsigned INT8	redraw = TRUE;

	unsigned int	n = NumFormulas;

	tDispFormula	Formulas_Copy[NumFormulas+1];
	memcpy(Formulas_Copy, Formulas, (NumFormulas+1)*sizeof(tDispFormula));
	
	/* unshow all empty formulas */
	while(n--) Formulas_Copy[n].Show = (Formulas_Copy[n].Show && *(Formulas_Copy[n].Formula->Formula));

	while (1)
	{
		if(redraw) {
			/* allow to switch between xmin, xmax, ymin, ymax from number editor */
			while(redraw) {
				SetupMode_Draw(x_min_copy, x_max_copy, y_min_copy, y_max_copy, Formulas_Copy);
				switch(redraw)
				{
				case 0x01:
					redraw = FALSE;
					break;
				case 0x02:
					redraw = EditNum(&x_min_copy, SETUPMODE_NUM_YPOS+0*SETUPMODE_NUM_YSPACE+1);
					break;
				case 0x04:
					redraw = EditNum(&x_max_copy, SETUPMODE_NUM_YPOS+1*SETUPMODE_NUM_YSPACE+1);
					break;
				case 0x08:
					redraw = EditNum(&y_min_copy, SETUPMODE_NUM_YPOS+2*SETUPMODE_NUM_YSPACE+1);
					break;
				case 0x10:
					redraw = EditNum(&y_max_copy, SETUPMODE_NUM_YPOS+3*SETUPMODE_NUM_YSPACE+1);
					break;
				}
			}
			SetupMode_TchInit();
			SetupMode_Draw(x_min_copy, x_max_copy, y_min_copy, y_max_copy, Formulas_Copy);
		}

		LibTchWait(&tsts);

		switch (tsts.obj)
		{
			/****************************************************************************************/
			/* Call of EditNum() will destroy touch information										*/
		case OBJ_XMIN:
			if (LibIconClick(&TXMinIcon, &tsts))
			{
				redraw = EditNum(&x_min_copy, SETUPMODE_NUM_YPOS+0*SETUPMODE_NUM_YSPACE+1);
			}
			break;
			
		case OBJ_XMAX:
			if (LibIconClick(&TXMaxIcon, &tsts))
			{
				redraw = EditNum(&x_max_copy, SETUPMODE_NUM_YPOS+1*SETUPMODE_NUM_YSPACE+1); 
			}
			break;
			
		case OBJ_YMIN:
			if (LibIconClick(&TYMinIcon, &tsts))
			{
				redraw = EditNum(&y_min_copy, SETUPMODE_NUM_YPOS+2*SETUPMODE_NUM_YSPACE+1);
			}
			break;
			
		case OBJ_YMAX:
			if (LibIconClick(&TYMaxIcon, &tsts))
			{					
				redraw = EditNum(&y_max_copy, SETUPMODE_NUM_YPOS+3*SETUPMODE_NUM_YSPACE+1);
			}
			break;
			
		case OBJ_SHOWF1:
		case OBJ_SHOWF2:
		case OBJ_SHOWF3:
			n = tsts.obj - OBJ_SHOWF1;
			if (LibIconClick(TIconShowF+n, &tsts))
			{
				Formulas_Copy[n].Show=!Formulas_Copy[n].Show;
				LibPutFarData(SETUPMODE_FSHOW_XPOS, SETUPMODE_FM_YPOS+(n+1)*SETUPMODE_FM_YSPACE, (Formulas_Copy[n].Show)?17:16);
				LibPutDisp();
			}
			break;
			
		case OBJ_DRAWFDX1:
		case OBJ_DRAWFDX2:
		case OBJ_DRAWFDX3:
			n = tsts.obj - OBJ_DRAWFDX1;
			if (LibIconClick(TIconDrawFDX+n, &tsts))
			{
				Formulas_Copy[n].Drawfx=!Formulas_Copy[n].Drawfx;
				LibPutFarData(SETUPMODE_FDX_XPOS, SETUPMODE_FM_YPOS+(n+1)*SETUPMODE_FM_YSPACE, (Formulas_Copy[n].Drawfx)?16:17);
				LibPutDisp();
			}
			break;
			
		case OBJ_SAVE:
			if (LibIconClick( &TSaveIcon, &tsts))
			{
				if (SetDisplayValues(x_min_copy, x_max_copy, y_min_copy, y_max_copy)) {
					memcpy(Formulas, Formulas_Copy, (NumFormulas+1)*sizeof(tDispFormula));
					SaveState();
					return (MODE_PLOT);
				}
				LibPutMsgDlg3("min cannot be greater\rthan or equal to max");
			}
			break;
		case OBJ_OK:
			if (LibIconClick( &TOKIcon, &tsts))
			{
				if (SetDisplayValues(x_min_copy, x_max_copy, y_min_copy, y_max_copy)) {
					memcpy(Formulas, Formulas_Copy, (NumFormulas+1)*sizeof(tDispFormula));
					return (MODE_PLOT);
				}
				LibPutMsgDlg3("min cannot be greater\rthan or equal to max");
			}
			break;
			
		case OBJ_CLR:
			if (LibIconClick(&TClrIcon, &tsts))
			{
				x_min_copy = DEF_XMIN;
				x_max_copy = DEF_XMAX;
				y_min_copy = DEF_YMIN;
				y_max_copy = DEF_YMAX;
				redraw = TRUE;
			}
			break;
			
		case OBJ_OWNESC:
			if(LibIconClick(&TEscIcon, &tsts))
				return (MODE_PLOT);
			break;
			
		case OBJ_HIC_ESC:
			return (MODE_EXIT);
		}

	}
}



/*****************************************************************************************************
*****************************************************************************************************
******    ******    ****        ****    **    ******    *********************************************
*******    ****    ****  ******  ****  ****   *******  **********************************************
*******  *  **  *  ****  ******  ****  ****    ******  **********************************************
*******  **    **  ****  ******  ****  ****  *  *****  **********************************************
*******  ***  ***  ****  ******  ****  ****  **  ****  **********************************************
*******  ********  ****          ****  ****  ***  ***  **********************************************
*******  ********  ****  ******  ****  ****  ****  **  **********************************************
*******  ********  ****  ******  ****  ****  *****  *  **********************************************
*******  ********  ****  ******  ****  ****  ******    **********************************************
******    ******    **    ****    **    **    ******    *********************************************
*****************************************************************************************************
*****************************************************************************************************/

void main (void)
{
	InitPVPlot();
	
	while (1)
	{
		switch (mode)
		{
		case MODE_EDIT:
			mode = EditMode_Run();
			break;
			
		case MODE_PLOT:
			mode = PlotMode_Run();
			break;
			
		case MODE_SETUP:
			mode = SetupMode_Run();
			break;
			
		default:
			SaveState();
			LibJumpMenu();
			break;
		}
	}
}
