/********************************************************************************
* Plotter grid for drawing function or data plots								*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* Grid.h																		*
*																				*
*																				*
* Version:	V1.03																*
* Date:		09.11.2002															*
********************************************************************************/


#ifndef _GRAPHLIB_H
#define _GRAPHLIB_H

#include "modeldef.h"


/* define for fixed memory Grid instead of drawing Grid dynamically on axis limits */
#define MEMORYGRID
#undef	MEMORYGRID


/* type for Display and Graph coordinates */
#define GRAPHCOORDTYPE	INT16


/**********************************************************************************
 * Display Constants/Limits for grid
 **********************************************************************************/
#define GRID_DX			144			/* BX-AX, width of grid */
#define GRID_DX2		(GRID_DX/2)		/* (BX-AX)/2 */

#define GRID_DY			136			/* BY-AY, height of grid */
#define GRID_DY2		(GRID_DY/2)		/* (BY-AY)/2 */

#define GRID_AXM		-16239		/* less than half of GRAPHCOORDTYPE range, else sub leads to overflow in line */
#define GRID_BXM		16239
#define GRID_AYM		-16239		/* less than half of GRAPHCOORDTYPE range, else sub leads to overflow in line  */
#define GRID_BYM		16239

#define	x_div		3			/* half Grid resolution and small step movement */
#define	y_div		3


/* default visible interval */
#define DEF_XMIN	-5.0
#define	DEF_XMAX	5.0
#define	DEF_YMIN	-5.0
#define	DEF_YMAX	5.0


/* Macros for checking coordinates in graph limits */
#define	INGRIDRANGEX(x)			(((x) >= 0) && ((x) <= GRID_DX))
#define	INGRIDRANGEY(y)			(((y) >= 0) && ((y) <= GRID_DY))
#define	INGRIDRANGE(x,y)		(INGRIDRANGEX(x) && INGRIDRANGEY(y))

/* Macro for calculating Number of Bytes in one Graphline */
#define XBYTES(x)				((x)/8+((((x)/8)%8)?1:0))

/* Macro for calculation Graphic Size in Byte */
#define GRAPHICSIZE(x,y)		(4+XBYTES((x))*(y))

/* definition of x- and y-range, set with SetDisplayValues for save handling */
typedef struct TGRIDLIMITS
{
	double	x_min;
	double	x_max;
	double	y_min;
	double	y_max;
	double	x_diff;
	double	y_diff;
} tGridLimits;


/********************************************************************************
 * exported data
 ********************************************************************************/
/* x- and y-range, actual and last values */
extern tGridLimits	GridLimits;

extern double			GridxValues[];		/* Array of x-Values for each grid point */

extern double			GridxValuesExt[];	/* Array of x-Values for each grid point + 2 additional */


#ifdef MEMORYGRID

/********************************************************************************
 * exported data
 ********************************************************************************/
extern byte FAR			GraphMem[];		/* memory for drawing graph */

#else /* MEMORYGRID */

/* Macro for calculating number of words in GraphMem, for faster clearing graphics area */
#define GRAPHSIZE(x,y)			((XBYTES((x))*(y))/2+(((XBYTES((x))*(y))%2)?1:0))

typedef struct TGRAPHMEM
{
	word	dx;
	word	dy;
	word	Mem[GRAPHSIZE(GRID_DX+1,GRID_DY+1)];
} tGraphMem;


/********************************************************************************
 * exported data
 ********************************************************************************/
extern tGraphMem FAR	GraphMem;		/* memory for drawing graph */

#endif /* MEMORYGRID */


/********************************************************************************
 * exported functions
 ********************************************************************************/
/* Calc Adress of byte with coordinate dot in */
extern byte FAR			*CalcXYMemAdress (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y);
/* Calc Values in GridxValues, return number of display points */
extern GRAPHCOORDTYPE	CalcGridxValues(tGridLimits *GridLimits);
/* Calc Values in GridxValues, return number of display points + one aditional point before and after limits */
extern GRAPHCOORDTYPE	CalcGridxValuesExt(tGridLimits *GridLimits);
/* init the x- and y-range to show in the plot for the first time and clear LIFO */
extern void				InitDisplayValues (double x_min, double x_max, double y_min, double y_max);
/* set the x- and y-range to show in the plot, saves old range in LIFO */
extern INT8				SetDisplayValues (double x_min, double x_max, double y_min, double y_max);
/* restore last range used */
extern INT8				RestoreOldDisplayValues (void);
/* convert graph coordinate to Display pixel value, relative in the Gridarea */
extern GRAPHCOORDTYPE	gtopx (double x);
extern GRAPHCOORDTYPE	gtopy (double y);
/* convert pixel coordinate to graph value, relative in the Gridarea */
extern double			ptogx (GRAPHCOORDTYPE X, GRAPHCOORDTYPE gxpos);
extern double			ptogy (GRAPHCOORDTYPE Y, GRAPHCOORDTYPE gypos);

/* init the grid memory, use before any other drawing function!! */
extern void				InitGrid (void);

/* clear GraphMem */
extern void				ClearGrid (void);
/* clear GraphMem and draw the grid in GraphMem */
extern void				DrawGrid (void);
/* draw scaling in GraphMem */
extern void				DrawAxis (void);

/* display GraphMem on the screen at x, y after all drawing is done in GraphMem */
extern void				ShowGraph (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y);


/********************************************************************************
* Plot functions to draw in GraphMem											*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* Graph.h																		*
*																				*
*																				*
* Version:	V1.03																*
* Date:		28.10.2002															*
********************************************************************************/


/* definition of a graph point */
typedef struct TGRAPHPOINT {
	double x;
	double y;
} tGraphPoint;

/* definition of a graph to plot with PlotGraph */
typedef struct TGRAPH {
	unsigned INT16	NumPoints;
	tGraphPoint		*Points;
} tGraph;


/* definition of a graph point in integer */
typedef struct TGRAPHPOINTINT {
	GRAPHCOORDTYPE x;
	GRAPHCOORDTYPE y;
} tGraphPointINT;

/* definition of a graph to plot with PlotGraph */
typedef struct TGRAPHINT {
	unsigned INT16	NumPoints;
	tGraphPointINT	*Points;
} tGraphINT;


/********************************************************************************
 * exported data
 ********************************************************************************/
extern unsigned INT8	dashon;		/* for dashed draw, set to start new lines with a dot in dashmode */

/********************************************************************************
 * exported functions
 ********************************************************************************/
/* set a dot in GraphMem */
extern void		SetDot (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y);
/* clear a dot in GraphMem */
extern void		ClrDot (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y);
/* invert a dot in GraphMem */
extern void		InvDot (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y);
/* Get Dot State, result -1 out of range, 0 not set, 1 set */
extern INT8		GetDot (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y);

/* Modes for drawing */
#define DRAWMODESET			0		/* set the dots */
#define DRAWMODEDASH		1		/* dash a line */
#define DRAWMODEDASHINVERT	2		/* dash and invert the dots */
#define DRAWMODEINVERT		3		/* invert the dots */
#define DRAWMODEUNSET		4		/* clear the dots */

/* draw a line in GraphMem, dashed when mode == 1, use dashon to reset dashstate at the beginning */
extern void		LineClipped (GRAPHCOORDTYPE x1, GRAPHCOORDTYPE y1, GRAPHCOORDTYPE x2, GRAPHCOORDTYPE y2, INT8 mode);
/* draw a numberstring in GraphMem, only numbers are supported, vertical drawing when vertical <> 0 */
extern void		GridPut35Str (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y, char *c, INT8 vertical);

/* plot a graph in GraphMem */
extern void		PlotGraph (tGraph *Graph, INT8 mode);
/* plot a graph in GraphMem */
extern void		PlotGraphS (double *xValues, double *yValues, unsigned INT16 NumPoints, INT8 mode);
/* plot a graph in GraphMem, using integer coordinates of grid */
extern void		PlotGraphINT (tGraphINT *Graph, INT8 mode);
/* plot a graph in GraphMem, using integer coordinates of grid */
extern void		PlotGraphSINT (GRAPHCOORDTYPE *xValues, GRAPHCOORDTYPE *yValues, unsigned INT16 NumPoints, INT8 mode);

/* diplay a busy sign on the display */
extern void		DrawBusy (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y);

/* Draw little Cross in Graphmem, remove last cross if one exists, returns 1 if a change in Grid happend */
extern INT8		DrawCrossInGrid (GRAPHCOORDTYPE x, GRAPHCOORDTYPE y);

#endif /* _GRAPHLIB_H */
