/********************************************************************************
* Math Extention																*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* MathExt.h																		*
*																				*
* Math Extention routines for function evaluation and general use				*
*																				*
* ToDo:	-Build our own mathlib													*
*																				*
*																				*
* Version:	V2.05																*
* Date:		01.11.2002															*
********************************************************************************/

#ifndef _MATHEXT_H
#define _MATHEXT_H

#include "modeldef.h"

#include <math.h>

/* define to use ERRNO math error check instead of the convenient matherr */
#define NOMATHERRUSEERRNO

/* search for reasons to use matherr */
#ifdef matherr
#ifdef NOMATHERRUSEERRNO
#undef NOMATHERRUSEERRNO
#endif
#endif

#ifdef DOMAIN
#ifdef SING
#ifdef OVERFLOW
#ifdef UNDERFLOW
#ifdef TLOSS
#ifdef PLOSS

#ifdef NOMATHERRUSEERRNO
#undef NOMATHERRUSEERRNO
#endif

#endif
#endif
#endif
#endif
#endif
#endif


#ifdef NOMATHERRUSEERRNO
#include <errno.h>
#endif

/* struct for math error handler, just for test purpose on other machines */
#ifdef NOMATHERRUSEERRNO

typedef struct	MEXCEPTION
{
	int			type;
	char		*name;
	double		arg1;
	double		arg2;
	double		retval;
} mexception;

#else /* NOMATHERRUSEERRNO */

#ifdef NOCASIOPVCOMPILER
struct exception {
	int			type;
	char		*name;
	double		arg1;
	double		arg2;
	double		retval;
};
#endif /* NOCASIOPVCOMPILER */

#endif /* NOMATHERRUSEERRNO */


/* BOOL */
#define FALSE				0
#define TRUE				1


/* Constants, try to get most of the possible accuracy of the compiler */
/* unfortunately the constants in the mathlib are weak in accuracy */
/* even in GNU newlib are wrong constants, why? historically? */
/* we need our own mathlib it seems... */
#define NULLCONST			0.0
/* #define PICONST			3.1415926535897932384626433832795028841971693993751058209749445923 */
#define PICONST				3.1415926535897932384626433832795028842
/* #define EULERCONST		2.718281828459045235360287471352662497757247093699959574966967627724 */
#define EULERCONST			2.7182818284590452353602874713526624978
#define HALFCONST			0.5
#define FOURTHCONST			0.25
#define THRFOURCONST		0.75


/* Math Exception, used internaly */
#ifdef NOMATHERRUSEERRNO

extern mexception			matherrordescr;
extern int					matherr (mexception *e);	/* math error handler, used internaly */

#else /* NOMATHERRUSEERRNO */

extern struct exception		matherrordescr;
extern int					matherr (struct exception *e);	/* math error handler, used internaly */

#endif /* NOMATHERRUSEERRNO */


extern char					LastMathErrorMsg[];				/* memory for math error message */
extern char					MathError;						/* flag for error in math functions */


/* exported Math constants in memory */
extern const double cNull;
extern const double cPi;
extern const double cEuler;
extern const double cHalf;
extern const double cFourth;
extern const double cThrfour;


/* exported Math Extention Functions, some needed just for Parser */
extern double	addv (double a, double b);		/* a+b */
extern double	subv (double a, double b);		/* a-b */
extern double	mulv (double a, double b);		/* a*b */
extern double	divv (double a, double b);		/* a/b */

extern double	pow10 (double a);				/* 10^a */
extern double	logxy (double a, double b);		/* log(a)/log(b) */
extern double	round (double a);				/* round a to nearest integer */
extern double	sqr (double a);					/* a*a */
extern double	trunc (double a);				/* round to integer closer to zero */ 
extern double	thre (double a);				/* a*a*a */
extern double	fak (double n);					/* n! */
extern double	rootxy (double a, double b);	/* a^(1/b) */
extern double	root3 (double a);				/* a^(1/3) */

/* exported functions */
extern char		*BuildMathErrorMsg (char FAR *Message);	/* build an error message from last math error happened */


/********************************************************************************
* Parser Errors																	*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* ParseErr.h																	*
*																				*
* Parser Error numbers and Messages												*
*																				*
* Version:	V2.02																*
* Date:		01.11.2002															*
********************************************************************************/


/* Parser ERROR numbers */
#define ERR_OK				0						/* last was ok */
#define ERR_READY			1						/* to show ready, not used */
#define ERR_MATH			2						/* math error */
#define ERR_MEMORY			3						/* parser memory error */
#define ERR_SYNTAX			4						/* syntax error in formula */
#define ERR_UNSUPPORTED		5						/* unsupported function found */
#define ERR_NOFORMULA		6						/* no formula in string */
#define ERR_EVALUATION		7						/* error during formula evaluation */
#define ERR_PAREN			8						/* error in paren usage */
#define ERR_KOMMA			9						/* error in komma usage */
#define ERR_BOUNDS			10						/* bounds wrong in zero crossing search */
#define ERR_ENDLESS			11						/* endless search for zero crossing */
#define ERR_ACCURACY		12						/* accuracy problems in SearchZero */
#define ERR_HOPPING			13						/* Hopping problems in SearchZero */
#define ERR_DX				14						/* dx is zero in derivative */


#define ERRMSG_MAX_LEN		79						/* maximum lenght for error messages */


/* exported Variables */
extern const char FAR		*ErrorMsgs[];			/* error table of messages, used internaly */

#endif	/* _MATHEXT_H */