/********************************************************************************
* table oriented Parser for mathematical formulas								*
*																				*
* by Ansgar Schfer, (c)2002													*
*																				*
* Parser.h																		*
*																				*
* defines exported symbols, functions and variables								*
*																				*
* for supported functions see ParseTbl.h										*
*																				*
* Version:	V2.04																*
* Date:		01.11.2002															*
********************************************************************************/

#ifndef _PARSER_H
#define _PARSER_H

#include "modeldef.h"

#include "MathExt.h"


/* Limits */
#define FORMULA_MAX_LEN		167				/* maximum number of characters allowed in a formula string */
#define	INTERNALBSIZE		FORMULA_MAX_LEN*3/2 /* internal data longer cause of -x -> 0-x */
#define	INTERNALISIZE		FORMULA_MAX_LEN*3/2

/* type of a formula */
typedef struct TFORMULA {
	INT8	FormulaChanged;					/* Flag for change in formula, set to true, when changing formula string!!! */
	INT8	IsFormula;						/* Flag for formula in x, set to true if x is found after parsing */
	char	Formula[FORMULA_MAX_LEN+1];		/* Formula string to put your formula in */
	double	dx;								/* dx for derivative calculations, set by InitFormula to a good value */
	INT8	ParserError;					/* last error number of parser */
	char	LastErrorMsg[ERRMSG_MAX_LEN+1];	/* last error message of parser */

	/* now parser internal data, do not use or change */
	void FAR			*Internala;
	unsigned INT16		Internalb;
	struct tInternalc {double (*a)(void);double b;} Internalc[INTERNALISIZE];
	void FAR			*Internald;
	double				Internale;

	struct tInternalf {void FAR *a; double b;} Internalf[INTERNALBSIZE];
	unsigned INT16		Internalg;
	unsigned INT16		Internalh;
	INT16				Internali;
	INT16				Internalj;
	INT8				Internalk;
} tFormula;


/* Global exported Variables */
extern double			ZeroAccept;					/* Value indicating a zero, numerical accuracy... */
extern double			GoodDerivDX;				/* Good value for dx in derivative or sec. derivative */

/* Exported Funktions */
extern void				InitFormula (tFormula FAR *Formula);	/* initialize Formula state, use after declaration of tFormula variable */

/* parse and evaluate formula to x */
extern double			Parse (tFormula FAR *Formula, double xvalue);
/* numericaly calculate derivative of formula to x with dx */
extern double			Derivative (tFormula FAR *Formula, double xvalue);
/* numericaly calculate second derivative of formula to x with dx, big error possible */
extern double			SecDerivative (tFormula FAR *Formula, double xvalue);
/* numericaly search a zero crossing of formula in interval [xstart,xend] */
extern double			SearchxZero (tFormula FAR *Formula, double xstart, double xend);
/* numericaly search a zero crossing of (numerical) derivative of formula in interval [xstart,xend] */
extern double			SearchxZeroDeriv (tFormula FAR *Formula, double xstart, double xend);

/* deliver function name of function in Parsetable, useful for own keyboard... */
extern char FAR			*FunctionName (INT16 index);

#endif	/* _PARSER_H */
